@@ -1,18 +1,22 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE examples.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework examples.
+   Copyright (c) Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
-   To use, copy, modify, and/or distribute this software for any purpose with or
+   to use, copy, modify, and/or distribute this software for any purpose with or
    without fee is hereby granted provided that the above copyright notice and
    this permission notice appear in all copies.
 
-   THE SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES,
-   WHETHER EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR
-   PURPOSE, ARE DISCLAIMED.
+   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+   REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+   AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
+   INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+   PERFORMANCE OF THIS SOFTWARE.
 
   ==============================================================================
 */
@@ -49,11 +53,18 @@
 #include "../Assets/DemoUtilities.h"
 
 //==============================================================================
-class DemoBackgroundThread  : public ThreadWithProgressWindow
+struct MessageBoxOwnerComponent : public Component
+{
+    ScopedMessageBox messageBox;
+};
+
+//==============================================================================
+class DemoBackgroundThread final : public ThreadWithProgressWindow
 {
 public:
-    DemoBackgroundThread()
-        : ThreadWithProgressWindow ("busy doing some important things...", true, true)
+    explicit DemoBackgroundThread (MessageBoxOwnerComponent& comp)
+        : ThreadWithProgressWindow ("busy doing some important things...", true, true),
+          owner (&comp)
     {
         setStatusMessage ("Getting ready...");
     }
@@ -91,21 +102,26 @@ public:
     {
         const String messageString (userPressedCancel ? "You pressed cancel!" : "Thread finished ok!");
 
-        AlertWindow::showAsync (MessageBoxOptions()
-                                  .withIconType (MessageBoxIconType::InfoIcon)
-                                  .withTitle ("Progress window")
-                                  .withMessage (messageString)
-                                  .withButton ("OK"),
-                                nullptr);
+        if (owner != nullptr)
+        {
+            owner->messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                  .withIconType (MessageBoxIconType::InfoIcon)
+                                                                  .withTitle ("Progress window")
+                                                                  .withMessage (messageString)
+                                                                  .withButton ("OK"),
+                                                              nullptr);
+        }
 
         // ..and clean up by deleting our thread object..
         delete this;
     }
+
+    Component::SafePointer<MessageBoxOwnerComponent> owner;
 };
 
 
 //==============================================================================
-class DialogsDemo  : public Component
+class DialogsDemo final : public MessageBoxOwnerComponent
 {
 public:
     enum DialogType
@@ -114,7 +130,7 @@ public:
         warningAlertWindow,
         infoAlertWindow,
         questionAlertWindow,
-        okCancelAlertWindow,
+        yesNoCancelAlertWindow,
         extraComponentsAlertWindow,
         calloutBoxWindow,
         progressWindow,
@@ -140,7 +156,7 @@ public:
                                   "Alert Window With Warning Icon",
                                   "Alert Window With Info Icon",
                                   "Alert Window With Question Icon",
-                                  "OK Cancel Alert Window",
+                                  "Yes No Cancel Alert Window",
                                   "Alert Window With Extra Components",
                                   "CalloutBox",
                                   "Thread With Progress Window",
@@ -168,18 +184,19 @@ public:
 
         setSize (500, 500);
 
-        RuntimePermissions::request (RuntimePermissions::readExternalStorage,
-                                     [] (bool granted)
-                                     {
-                                         if (! granted)
-                                             AlertWindow::showAsync (MessageBoxOptions()
-                                                                       .withIconType (MessageBoxIconType::WarningIcon)
-                                                                       .withTitle ("Permissions warning")
-                                                                       .withMessage ("External storage access permission not granted, some files"
-                                                                                     " may be inaccessible.")
-                                                                       .withButton ("OK"),
-                                                                     nullptr);
-                                     });
+        RuntimePermissions::request (RuntimePermissions::readExternalStorage, [ptr = Component::SafePointer (this)] (bool granted)
+        {
+            if (granted || ptr == nullptr)
+               return;
+
+            ptr->messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                .withIconType (MessageBoxIconType::WarningIcon)
+                                                                .withTitle ("Permissions warning")
+                                                                .withMessage ("External storage access permission not granted, some files"
+                                                                              " may be inaccessible.")
+                                                                .withButton ("OK"),
+                                                            nullptr);
+        });
     }
 
     //==============================================================================
@@ -214,49 +231,51 @@ private:
     OwnedArray<TextButton> windowButtons;
     ToggleButton nativeButton;
 
-    struct AlertBoxResultChosen
+    auto getAlertBoxResultChosen()
     {
-        void operator() (int result) const noexcept
+        return [ptr = Component::SafePointer (this)] (int result)
         {
-            AlertWindow::showAsync (MessageBoxOptions()
-                                      .withIconType (MessageBoxIconType::InfoIcon)
-                                      .withTitle ("Alert Box")
-                                      .withMessage ("Result code: " + String (result))
-                                      .withButton ("OK"),
-                                    nullptr);
-        }
-    };
+            if (ptr != nullptr)
+                ptr->messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                    .withIconType (MessageBoxIconType::InfoIcon)
+                                                                    .withTitle ("Alert Box")
+                                                                    .withMessage ("Result code: " + String (result))
+                                                                    .withButton ("OK"),
+                                                                nullptr);
+        };
+    }
 
-    struct AsyncAlertBoxResultChosen
+    auto getAsyncAlertBoxResultChosen()
     {
-        void operator() (int result) const noexcept
+        return [ptr = Component::SafePointer (this)] (int result)
         {
-            auto& aw = *demo.asyncAlertWindow;
+            if (ptr == nullptr)
+                return;
+
+            auto& aw = *ptr->asyncAlertWindow;
 
             aw.exitModalState (result);
             aw.setVisible (false);
 
             if (result == 0)
             {
-                AlertBoxResultChosen{} (result);
+                ptr->getAlertBoxResultChosen() (result);
                 return;
             }
 
             auto optionIndexChosen = aw.getComboBoxComponent ("option")->getSelectedItemIndex();
             auto text = aw.getTextEditorContents ("text");
 
-            AlertWindow::showAsync (MessageBoxOptions()
-                                      .withIconType (MessageBoxIconType::InfoIcon)
-                                      .withTitle ("Alert Box")
-                                      .withMessage ("Result code: " + String (result) + newLine
-                                                    + "Option index chosen: " + String (optionIndexChosen) + newLine
-                                                    + "Text: " + text)
-                                      .withButton ("OK"),
-                                    nullptr);
-        }
-
-        DialogsDemo& demo;
-    };
+            ptr->messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                .withIconType (MessageBoxIconType::InfoIcon)
+                                                                .withTitle ("Alert Box")
+                                                                .withMessage ("Result code: " + String (result) + newLine
+                                                                              + "Option index chosen: " + String (optionIndexChosen) + newLine
+                                                                              + "Text: " + text)
+                                                                .withButton ("OK"),
+                                                            nullptr);
+        };
+    }
 
     void showWindow (Component& button, DialogType type)
     {
@@ -268,16 +287,19 @@ private:
             if (type == infoAlertWindow)      icon = MessageBoxIconType::InfoIcon;
             if (type == questionAlertWindow)  icon = MessageBoxIconType::QuestionIcon;
 
-            AlertWindow::showMessageBoxAsync (icon, "This is an AlertWindow",
-                                              "And this is the AlertWindow's message. Blah blah blah blah blah blah blah blah blah blah blah blah blah.",
-                                              "OK");
+            auto options = MessageBoxOptions::makeOptionsOk (icon,
+                                                             "This is an AlertWindow",
+                                                             "And this is the AlertWindow's message. "
+                                                             "Blah blah blah blah blah blah blah blah blah blah blah blah blah.");
+            messageBox = AlertWindow::showScopedAsync (options, nullptr);
         }
-        else if (type == okCancelAlertWindow)
+        else if (type == yesNoCancelAlertWindow)
         {
-            AlertWindow::showOkCancelBox (MessageBoxIconType::QuestionIcon, "This is an ok/cancel AlertWindow",
-                                          "And this is the AlertWindow's message. Blah blah blah blah blah blah blah blah blah blah blah blah blah.",
-                                          {}, {}, {},
-                                          ModalCallbackFunction::create (AlertBoxResultChosen{}));
+            auto options = MessageBoxOptions::makeOptionsYesNoCancel (MessageBoxIconType::QuestionIcon,
+                                                                      "This is a yes/no/cancel AlertWindow",
+                                                                      "And this is the AlertWindow's message. "
+                                                                      "Blah blah blah blah blah blah blah blah blah blah blah blah blah.");
+            messageBox = AlertWindow::showScopedAsync (options, getAlertBoxResultChosen());
         }
         else if (type == calloutBoxWindow)
         {
@@ -301,13 +323,13 @@ private:
             asyncAlertWindow->addButton ("OK",     1, KeyPress (KeyPress::returnKey, 0, 0));
             asyncAlertWindow->addButton ("Cancel", 0, KeyPress (KeyPress::escapeKey, 0, 0));
 
-            asyncAlertWindow->enterModalState (true, ModalCallbackFunction::create (AsyncAlertBoxResultChosen { *this }));
+            asyncAlertWindow->enterModalState (true, ModalCallbackFunction::create (getAsyncAlertBoxResultChosen()));
         }
         else if (type == progressWindow)
         {
             // This will launch our ThreadWithProgressWindow in a modal state. (Our subclass
             // will take care of deleting the object when the task has finished)
-            (new DemoBackgroundThread())->launchThread();
+            (new DemoBackgroundThread (*this))->launchThread();
         }
         else if (type >= loadChooser && type <= saveChooser)
         {
@@ -318,9 +340,10 @@ private:
                 fc.reset (new FileChooser ("Choose a file to open...", File::getCurrentWorkingDirectory(),
                                            "*", useNativeVersion));
 
-                fc->launchAsync (FileBrowserComponent::canSelectMultipleItems | FileBrowserComponent::openMode
-                                     | FileBrowserComponent::canSelectFiles,
-                                 [] (const FileChooser& chooser)
+                fc->launchAsync (FileBrowserComponent::canSelectMultipleItems
+                                 | FileBrowserComponent::openMode
+                                 | FileBrowserComponent::canSelectFiles,
+                                 [this] (const FileChooser& chooser)
                                  {
                                      String chosen;
                                      auto results = chooser.getURLResults();
@@ -329,12 +352,12 @@ private:
                                          chosen << (result.isLocalFile() ? result.getLocalFile().getFullPathName()
                                                                          : result.toString (false)) << "\n";
 
-                                     AlertWindow::showAsync (MessageBoxOptions()
-                                                               .withIconType (MessageBoxIconType::InfoIcon)
-                                                               .withTitle ("File Chooser...")
-                                                               .withMessage ("You picked: " + chosen)
-                                                               .withButton ("OK"),
-                                                             nullptr);
+                                     messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                                    .withIconType (MessageBoxIconType::InfoIcon)
+                                                                                    .withTitle ("File Chooser...")
+                                                                                    .withMessage ("You picked: " + chosen)
+                                                                                    .withButton ("OK"),
+                                                                                nullptr);
                                  });
             }
             else if (type == loadWithPreviewChooser)
@@ -344,9 +367,10 @@ private:
                 fc.reset (new FileChooser ("Choose an image to open...", File::getCurrentWorkingDirectory(),
                                            "*.jpg;*.jpeg;*.png;*.gif", useNativeVersion));
 
-                fc->launchAsync (FileBrowserComponent::openMode | FileBrowserComponent::canSelectFiles
-                                    | FileBrowserComponent::canSelectMultipleItems,
-                                 [] (const FileChooser& chooser)
+                fc->launchAsync (FileBrowserComponent::openMode
+                                 | FileBrowserComponent::canSelectFiles
+                                 | FileBrowserComponent::canSelectMultipleItems,
+                                 [this] (const FileChooser& chooser)
                                  {
                                      String chosen;
                                      auto results = chooser.getURLResults();
@@ -355,12 +379,12 @@ private:
                                          chosen << (result.isLocalFile() ? result.getLocalFile().getFullPathName()
                                                                          : result.toString (false)) << "\n";
 
-                                     AlertWindow::showAsync (MessageBoxOptions()
-                                                               .withIconType (MessageBoxIconType::InfoIcon)
-                                                               .withTitle ("File Chooser...")
-                                                               .withMessage ("You picked: " + chosen)
-                                                               .withButton ("OK"),
-                                                             nullptr);
+                                     messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                                    .withIconType (MessageBoxIconType::InfoIcon)
+                                                                                    .withTitle ("File Chooser...")
+                                                                                    .withMessage ("You picked: " + chosen)
+                                                                                    .withButton ("OK"),
+                                                                                nullptr);
                                  },
                                  &imagePreview);
             }
@@ -385,7 +409,7 @@ private:
                                            "*",  useNativeVersion));
 
                 fc->launchAsync (FileBrowserComponent::saveMode | FileBrowserComponent::canSelectFiles,
-                                 [fileToSave] (const FileChooser& chooser)
+                                 [this, fileToSave] (const FileChooser& chooser)
                                  {
                                      auto result = chooser.getURLResult();
                                      auto name = result.isEmpty() ? String()
@@ -402,19 +426,19 @@ private:
 
                                          if (wi.get() != nullptr && wo.get() != nullptr)
                                          {
-                                             auto numWritten = wo->writeFromInputStream (*wi, -1);
-                                             jassertquiet (numWritten > 0);
+                                             [[maybe_unused]] auto numWritten = wo->writeFromInputStream (*wi, -1);
+                                             jassert (numWritten > 0);
                                              wo->flush();
                                          }
                                      }
                                    #endif
 
-                                     AlertWindow::showAsync (MessageBoxOptions()
-                                                               .withIconType (MessageBoxIconType::InfoIcon)
-                                                               .withTitle ("File Chooser...")
-                                                               .withMessage ("You picked: " + name)
-                                                               .withButton ("OK"),
-                                                             nullptr);
+                                     messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                                    .withIconType (MessageBoxIconType::InfoIcon)
+                                                                                    .withTitle ("File Chooser...")
+                                                                                    .withMessage ("You picked: " + name)
+                                                                                    .withButton ("OK"),
+                                                                                nullptr);
                                  });
             }
             else if (type == directoryChooser)
@@ -425,35 +449,37 @@ private:
                                            useNativeVersion));
 
                 fc->launchAsync (FileBrowserComponent::openMode | FileBrowserComponent::canSelectDirectories,
-                                 [] (const FileChooser& chooser)
+                                 [this] (const FileChooser& chooser)
                                  {
                                      auto result = chooser.getURLResult();
                                      auto name = result.isLocalFile() ? result.getLocalFile().getFullPathName()
                                                                       : result.toString (true);
 
-                                     AlertWindow::showAsync (MessageBoxOptions()
-                                                               .withIconType (MessageBoxIconType::InfoIcon)
-                                                               .withTitle ("File Chooser...")
-                                                               .withMessage ("You picked: " + name)
-                                                               .withButton ("OK"),
-                                                             nullptr);
+                                     messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                                    .withIconType (MessageBoxIconType::InfoIcon)
+                                                                                    .withTitle ("File Chooser...")
+                                                                                    .withMessage ("You picked: " + name)
+                                                                                    .withButton ("OK"),
+                                                                                nullptr);
                                  });
             }
         }
         else if (type == shareText)
         {
-            ContentSharer::getInstance()->shareText ("I love JUCE!",
-                                                     [] (bool success, const String& error)
-                {
-                    auto resultString = success ? String ("success") : ("failure\n (error: " + error + ")");
+            messageBox = ContentSharer::shareTextScoped ("I love JUCE!", [ptr = Component::SafePointer (this)] (bool success, const String& error)
+            {
+                if (ptr == nullptr)
+                    return;
 
-                    AlertWindow::showAsync (MessageBoxOptions()
-                                              .withIconType (MessageBoxIconType::InfoIcon)
-                                              .withTitle ("Sharing Text Result")
-                                              .withMessage ("Sharing text finished\nwith " + resultString)
-                                              .withButton ("OK"),
-                                            nullptr);
-                });
+                auto resultString = success ? String ("success") : ("failure\n (error: " + error + ")");
+
+                ptr->messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                    .withIconType (MessageBoxIconType::InfoIcon)
+                                                                    .withTitle ("Sharing Text Result")
+                                                                    .withMessage ("Sharing text finished\nwith " + resultString)
+                                                                    .withButton ("OK"),
+                                                                nullptr);
+            });
         }
         else if (type == shareFile)
         {
@@ -467,18 +493,20 @@ private:
                 Array<URL> urls;
                 urls.add (URL (fileToSave));
 
-                ContentSharer::getInstance()->shareFiles (urls,
-                    [] (bool success, const String& error)
-                    {
-                        auto resultString = success ? String ("success") : ("failure\n (error: " + error + ")");
+                messageBox = ContentSharer::shareFilesScoped (urls, [ptr = Component::SafePointer (this)] (bool success, const String& error)
+                {
+                    if (ptr == nullptr)
+                        return;
 
-                        AlertWindow::showAsync (MessageBoxOptions()
-                                                  .withIconType (MessageBoxIconType::InfoIcon)
-                                                  .withTitle ("Sharing Files Result")
-                                                  .withMessage ("Sharing files finished\nwith " + resultString)
-                                                  .withButton ("OK"),
-                                                nullptr);
-                    });
+                    auto resultString = success ? String ("success") : ("failure\n (error: " + error + ")");
+
+                    ptr->messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                        .withIconType (MessageBoxIconType::InfoIcon)
+                                                                        .withTitle ("Sharing Files Result")
+                                                                        .withMessage ("Sharing files finished\nwith " + resultString)
+                                                                        .withButton ("OK"),
+                                                                    nullptr);
+                });
 
             }
         }
@@ -495,19 +523,21 @@ private:
 
             Array<Image> images { myImage, myImage2 };
 
-            ContentSharer::getInstance()->shareImages (images,
-                                                       [] (bool success, const String& error)
-                                                       {
-                                                           String resultString = success ? String ("success")
-                                                                                         : ("failure\n (error: " + error + ")");
+            messageBox = ContentSharer::shareImagesScoped (images, nullptr, [ptr = Component::SafePointer (this)] (bool success, const String& error)
+            {
+                if (ptr == nullptr)
+                    return;
 
-                                                           AlertWindow::showAsync (MessageBoxOptions()
-                                                                                     .withIconType (MessageBoxIconType::InfoIcon)
-                                                                                     .withTitle ("Sharing Images Result")
-                                                                                     .withMessage ("Sharing images finished\nwith " + resultString)
-                                                                                     .withButton ("OK"),
-                                                                                   nullptr);
-                                                       });
+                String resultString = success ? String ("success")
+                                              : ("failure\n (error: " + error + ")");
+
+                ptr->messageBox = AlertWindow::showScopedAsync (MessageBoxOptions()
+                                                                    .withIconType (MessageBoxIconType::InfoIcon)
+                                                                    .withTitle ("Sharing Images Result")
+                                                                    .withMessage ("Sharing images finished\nwith " + resultString)
+                                                                    .withButton ("OK"),
+                                                                nullptr);
+            });
         }
     }
 
