@@ -1,18 +1,22 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE examples.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework examples.
+   Copyright (c) Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
-   To use, copy, modify, and/or distribute this software for any purpose with or
+   to use, copy, modify, and/or distribute this software for any purpose with or
    without fee is hereby granted provided that the above copyright notice and
    this permission notice appear in all copies.
 
-   THE SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES,
-   WHETHER EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR
-   PURPOSE, ARE DISCLAIMED.
+   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+   REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+   AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
+   INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+   PERFORMANCE OF THIS SOFTWARE.
 
   ==============================================================================
 */
@@ -50,7 +54,7 @@
 
 //==============================================================================
 /** Holds the various toggle buttons for the animation modes. */
-class ControllersComponent  : public Component
+class ControllersComponent final : public Component
 {
 public:
     ControllersComponent()
@@ -113,14 +117,14 @@ public:
 };
 
 //==============================================================================
-class GraphicsDemoBase  : public Component
+class GraphicsDemoBase : public Component
 {
 public:
     GraphicsDemoBase (ControllersComponent& cc, const String& name)
         : Component (name),
           controls (cc)
     {
-        displayFont = Font (Font::getDefaultMonospacedFontName(), 12.0f, Font::bold);
+        displayFont = FontOptions (Font::getDefaultMonospacedFontName(), 12.0f, Font::bold);
     }
 
     AffineTransform getTransform()
@@ -274,13 +278,13 @@ public:
 
     double lastRenderStartTime = 0.0, averageTimeMs = 0.0, averageActualFPS = 0.0;
     Image clipImage;
-    Font displayFont;
+    Font displayFont { FontOptions{} };
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (GraphicsDemoBase)
 };
 
 //==============================================================================
-class RectangleFillTypesDemo  : public GraphicsDemoBase
+class RectangleFillTypesDemo final : public GraphicsDemoBase
 {
 public:
     RectangleFillTypesDemo (ControllersComponent& cc)
@@ -316,7 +320,7 @@ public:
 };
 
 //==============================================================================
-class PathsDemo  : public GraphicsDemoBase
+class PathsDemo final : public GraphicsDemoBase
 {
 public:
     PathsDemo (ControllersComponent& cc, bool linear, bool radial)
@@ -377,7 +381,7 @@ public:
 };
 
 //==============================================================================
-class StrokesDemo  : public GraphicsDemoBase
+class StrokesDemo final : public GraphicsDemoBase
 {
 public:
     StrokesDemo (ControllersComponent& cc)
@@ -410,7 +414,7 @@ public:
 };
 
 //==============================================================================
-class ImagesRenderingDemo  : public GraphicsDemoBase
+class ImagesRenderingDemo final : public GraphicsDemoBase
 {
 public:
     ImagesRenderingDemo (ControllersComponent& cc, bool argb, bool tiled)
@@ -448,13 +452,13 @@ public:
 };
 
 //==============================================================================
-class GlyphsDemo  : public GraphicsDemoBase
+class GlyphsDemo final : public GraphicsDemoBase
 {
 public:
     GlyphsDemo (ControllersComponent& cc)
         : GraphicsDemoBase (cc, "Glyphs")
     {
-        glyphs.addFittedText ({ 20.0f }, "The Quick Brown Fox Jumped Over The Lazy Dog",
+        glyphs.addFittedText (FontOptions { 20.0f }, "The Quick Brown Fox Jumps Over The Lazy Dog",
                               -120, -50, 240, 100, Justification::centred, 2, 1.0f);
     }
 
@@ -468,7 +472,7 @@ public:
 };
 
 //==============================================================================
-class SVGDemo  : public GraphicsDemoBase
+class SVGDemo final : public GraphicsDemoBase
 {
 public:
     SVGDemo (ControllersComponent& cc)
@@ -513,7 +517,7 @@ public:
 };
 
 //==============================================================================
-class LinesDemo  : public GraphicsDemoBase
+class LinesDemo final : public GraphicsDemoBase
 {
 public:
     LinesDemo (ControllersComponent& cc)
@@ -576,8 +580,8 @@ public:
 };
 
 //==============================================================================
-class DemoHolderComponent  : public Component,
-                             private Timer
+class DemoHolderComponent final : public Component,
+                                  private Timer
 {
 public:
     DemoHolderComponent()
@@ -623,8 +627,8 @@ private:
 };
 
 //==============================================================================
-class TestListComponent   : public Component,
-                            private ListBoxModel
+class TestListComponent final : public Component,
+                                private ListBoxModel
 {
 public:
     TestListComponent (DemoHolderComponent& holder, ControllersComponent& controls)
@@ -695,7 +699,7 @@ private:
 };
 
 //==============================================================================
-class GraphicsDemo  : public Component
+class GraphicsDemo final : public Component
 {
 public:
     GraphicsDemo()
