@@ -1,18 +1,22 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE examples.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework examples.
+   Copyright (c) Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
-   To use, copy, modify, and/or distribute this software for any purpose with or
+   to use, copy, modify, and/or distribute this software for any purpose with or
    without fee is hereby granted provided that the above copyright notice and
    this permission notice appear in all copies.
 
-   THE SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES,
-   WHETHER EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR
-   PURPOSE, ARE DISCLAIMED.
+   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+   REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+   AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
+   INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+   PERFORMANCE OF THIS SOFTWARE.
 
   ==============================================================================
 */
@@ -59,7 +63,7 @@ static void showBubbleMessage (Component& targetComponent, const String& textToS
 /** To demonstrate how sliders can have custom snapping applied to their values,
     this simple class snaps the value to 50 if it comes near.
 */
-struct SnappingSlider  : public Slider
+struct SnappingSlider final : public Slider
 {
     double snapValue (double attemptedValue, DragMode dragMode) override
     {
@@ -74,8 +78,8 @@ struct SnappingSlider  : public Slider
 };
 
 /** A TextButton that pops up a colour chooser to change its colours. */
-class ColourChangeButton  : public TextButton,
-                            public ChangeListener
+class ColourChangeButton final : public TextButton,
+                                 public ChangeListener
 {
 public:
     ColourChangeButton()
@@ -112,7 +116,7 @@ public:
 };
 
 //==============================================================================
-struct SlidersPage  : public Component
+struct SlidersPage final : public Component
 {
     SlidersPage()
     {
@@ -135,47 +139,47 @@ struct SlidersPage  : public Component
         s->setTextValueSuffix (" rels");
 
         sliderArea.removeFromLeft (20);
-        auto horizonalSliderArea = sliderArea.removeFromLeft (180);
+        auto horizontalSliderArea = sliderArea.removeFromLeft (180);
 
         s = createSlider (true);
         s->setSliderStyle (Slider::LinearHorizontal);
         s->setTextBoxStyle (Slider::TextBoxLeft, false, 80, 20);
-        s->setBounds (horizonalSliderArea.removeFromTop (20));
+        s->setBounds (horizontalSliderArea.removeFromTop (20));
 
         s = createSlider (false);
         s->setSliderStyle (Slider::LinearHorizontal);
         s->setTextBoxStyle (Slider::NoTextBox, false, 0, 0);
-        horizonalSliderArea.removeFromTop (20);
-        s->setBounds (horizonalSliderArea.removeFromTop (20));
+        horizontalSliderArea.removeFromTop (20);
+        s->setBounds (horizontalSliderArea.removeFromTop (20));
         s->setPopupDisplayEnabled (true, false, this);
         s->setTextValueSuffix (" nuns required to change a lightbulb");
 
         s = createSlider (false);
         s->setSliderStyle (Slider::LinearHorizontal);
         s->setTextBoxStyle (Slider::TextEntryBoxPosition::TextBoxAbove, false, 70, 20);
-        horizonalSliderArea.removeFromTop (20);
-        s->setBounds (horizonalSliderArea.removeFromTop (50));
+        horizontalSliderArea.removeFromTop (20);
+        s->setBounds (horizontalSliderArea.removeFromTop (50));
         s->setPopupDisplayEnabled (true, false, this);
 
         s = createSlider (false);
         s->setSliderStyle (Slider::IncDecButtons);
         s->setTextBoxStyle (Slider::TextBoxLeft, false, 50, 20);
-        horizonalSliderArea.removeFromTop (20);
-        s->setBounds (horizonalSliderArea.removeFromTop (20));
+        horizontalSliderArea.removeFromTop (20);
+        s->setBounds (horizontalSliderArea.removeFromTop (20));
         s->setIncDecButtonsMode (Slider::incDecButtonsDraggable_Vertical);
 
         s = createSlider (false);
         s->setSliderStyle (Slider::Rotary);
         s->setRotaryParameters (MathConstants<float>::pi * 1.2f, MathConstants<float>::pi * 2.8f, false);
         s->setTextBoxStyle (Slider::TextBoxRight, false, 70, 20);
-        horizonalSliderArea.removeFromTop (15);
-        s->setBounds (horizonalSliderArea.removeFromTop (70));
+        horizontalSliderArea.removeFromTop (15);
+        s->setBounds (horizontalSliderArea.removeFromTop (70));
         s->setTextValueSuffix (" mm");
 
         s = createSlider (false);
         s->setSliderStyle (Slider::LinearBar);
-        horizonalSliderArea.removeFromTop (10);
-        s->setBounds (horizonalSliderArea.removeFromTop (30));
+        horizontalSliderArea.removeFromTop (10);
+        s->setBounds (horizontalSliderArea.removeFromTop (30));
         s->setTextValueSuffix (" gallons");
 
         sliderArea.removeFromLeft (20);
@@ -229,7 +233,7 @@ struct SlidersPage  : public Component
 
         for (int i = 8; i <= 11; ++i)
         {
-            auto* selectedSlider = sliders.getUnchecked(i);
+            auto* selectedSlider = sliders.getUnchecked (i);
             selectedSlider->setTextBoxStyle (Slider::NoTextBox, false, 0, 0);
             selectedSlider->getMaxValueObject().referTo (sharedValueMax);
             selectedSlider->getMinValueObject().referTo (sharedValueMin);
@@ -261,7 +265,7 @@ private:
 };
 
 //==============================================================================
-struct ButtonsPage   : public Component
+struct ButtonsPage final : public Component
 {
     ButtonsPage (bool isRunningComponentTransformDemo)
     {
@@ -461,7 +465,8 @@ private:
 
 
 //==============================================================================
-struct MiscPage   : public Component
+struct MiscPage final : public Component,
+                        private Timer
 {
     MiscPage()
     {
@@ -482,6 +487,21 @@ struct MiscPage   : public Component
             comboBox.addItem ("combo box item " + String (i), i);
 
         comboBox.setSelectedId (1);
+
+        addAndMakeVisible (linearProgressBar);
+        linearProgressBar.setStyle (ProgressBar::Style::linear);
+        linearProgressBar.setBounds (10, 115, 200, 24);
+
+        addAndMakeVisible (circularProgressBar);
+        circularProgressBar.setStyle (ProgressBar::Style::circular);
+        circularProgressBar.setBounds (10, 145, 200, 100);
+
+        startTimerHz (10);
+    }
+
+    ~MiscPage() override
+    {
+        stopTimer();
     }
 
     void lookAndFeelChanged() override
@@ -490,14 +510,41 @@ struct MiscPage   : public Component
         textEditor2.applyFontToAllText (textEditor2.getFont());
     }
 
+    void timerCallback() override
+    {
+        constexpr auto minValue = -0.2;
+        constexpr auto maxValue = 1.2;
+        constexpr auto maxIncrement = 0.05;
+
+        if (progress >= maxValue)
+            progress = minValue;
+        else
+            progress += Random::getSystemRandom().nextDouble() * maxIncrement;
+
+        if (isPositiveAndNotGreaterThan (progress, 1.0))
+        {
+            linearProgressBar.setPercentageDisplay (true);
+            circularProgressBar.setPercentageDisplay (true);
+        }
+        else
+        {
+            linearProgressBar.setTextToDisplay ("Linear progress bar");
+            circularProgressBar.setTextToDisplay ("Circular progress bar");
+        }
+    }
+
     TextEditor textEditor1,
                textEditor2  { "Password", (juce_wchar) 0x2022 };
 
     ComboBox comboBox  { "Combo" };
+
+    double progress { 0.0 };
+    ProgressBar linearProgressBar { progress };
+    ProgressBar circularProgressBar { progress };
 };
 
 //==============================================================================
-struct MenuPage   : public Component
+struct MenuPage final : public Component
 {
     MenuPage()
     {
@@ -568,7 +615,7 @@ struct MenuPage   : public Component
         addAndMakeVisible (customItemButton);
         customItemButton.onClick = [&]
         {
-            struct CustomComponent  : public PopupMenu::CustomComponent
+            struct CustomComponent final : public PopupMenu::CustomComponent
             {
                 CustomComponent (int widthIn, int heightIn, Colour backgroundIn)
                     : PopupMenu::CustomComponent (false),
@@ -647,7 +694,7 @@ struct MenuPage   : public Component
     {
         void drawPopupMenuColumnSeparatorWithOptions (Graphics& g,
                                                       const Rectangle<int>& bounds,
-                                                      const PopupMenu::Options& opt)
+                                                      const PopupMenu::Options& opt) override
         {
             if (auto* target = opt.getTargetComponent())
             {
@@ -665,7 +712,7 @@ struct MenuPage   : public Component
             }
         }
 
-        void drawPopupMenuBackgroundWithOptions (Graphics& g, int, int, const PopupMenu::Options& opt)
+        void drawPopupMenuBackgroundWithOptions (Graphics& g, int, int, const PopupMenu::Options& opt) override
         {
             if (auto* target = opt.getTargetComponent())
             {
@@ -674,7 +721,7 @@ struct MenuPage   : public Component
         }
 
         // Return the amount of space that should be left between popup menu columns.
-        int getPopupMenuColumnSeparatorWidthWithOptions (const PopupMenu::Options&)
+        int getPopupMenuColumnSeparatorWidthWithOptions (const PopupMenu::Options&) override
         {
             return 10;
         }
@@ -692,8 +739,8 @@ struct MenuPage   : public Component
 };
 
 //==============================================================================
-class ToolbarDemoComp   : public Component,
-                          private Slider::Listener
+class ToolbarDemoComp final : public Component,
+                              private Slider::Listener
 {
 public:
     ToolbarDemoComp()
@@ -759,7 +806,7 @@ private:
                customiseButton    { "Customise..." };
 
     //==============================================================================
-    class DemoToolbarItemFactory   : public ToolbarItemFactory
+    class DemoToolbarItemFactory final : public ToolbarItemFactory
     {
     public:
         DemoToolbarItemFactory() {}
@@ -882,7 +929,7 @@ private:
 
         // Demonstrates how to put a custom component into a toolbar - this one contains
         // a ComboBox.
-        class CustomToolbarComboBox : public ToolbarItemComponent
+        class CustomToolbarComboBox final : public ToolbarItemComponent
         {
         public:
             CustomToolbarComboBox (const int toolbarItemId)
@@ -934,8 +981,8 @@ private:
 /**
     This class shows how to implement a TableListBoxModel to show in a TableListBox.
 */
-class TableDemoComponent    : public Component,
-                              public TableListBoxModel
+class TableDemoComponent final : public Component,
+                                 public TableListBoxModel
 {
 public:
     TableDemoComponent()
@@ -1090,7 +1137,7 @@ public:
 
     String getText (const int columnNumber, const int rowNumber) const
     {
-        return dataList->getChildElement (rowNumber)->getStringAttribute ( getAttributeNameForColumnId(columnNumber));
+        return dataList->getChildElement (rowNumber)->getStringAttribute (getAttributeNameForColumnId (columnNumber));
     }
 
     void setText (const int columnNumber, const int rowNumber, const String& newText)
@@ -1109,7 +1156,7 @@ public:
 
 private:
     TableListBox table;     // the table component itself
-    Font font  { 14.0f };
+    Font font { FontOptions { 14.0f } };
 
     std::unique_ptr<XmlElement> demoData;  // This is the XML document loaded from the embedded file "demo table data.xml"
     XmlElement* columnList = nullptr;     // A pointer to the sub-node of demoData that contains the list of columns
@@ -1118,7 +1165,7 @@ private:
 
     //==============================================================================
     // This is a custom Label component, which we use for the table's editable text columns.
-    class EditableTextCustomComponent  : public Label
+    class EditableTextCustomComponent final : public Label
     {
     public:
         EditableTextCustomComponent (TableDemoComponent& td)  : owner (td)
@@ -1145,7 +1192,7 @@ private:
         {
             row = newRow;
             columnId = newColumn;
-            setText (owner.getText(columnId, row), dontSendNotification);
+            setText (owner.getText (columnId, row), dontSendNotification);
         }
 
         void paint (Graphics& g) override
@@ -1166,7 +1213,7 @@ private:
     //==============================================================================
     // This is a custom component containing a combo box, which we're going to put inside
     // our table's "rating" column.
-    class RatingColumnCustomComponent    : public Component
+    class RatingColumnCustomComponent final : public Component
     {
     public:
         RatingColumnCustomComponent (TableDemoComponent& td)  : owner (td)
@@ -1260,8 +1307,8 @@ private:
 };
 
 //==============================================================================
-class DragAndDropDemo  : public Component,
-                         public DragAndDropContainer
+class DragAndDropDemo final : public Component,
+                              public DragAndDropContainer
 {
 public:
     DragAndDropDemo()
@@ -1285,7 +1332,7 @@ public:
 
 private:
     //==============================================================================
-    struct SourceItemListboxContents  : public ListBoxModel
+    struct SourceItemListboxContents final : public ListBoxModel
     {
         // The following methods implement the necessary virtual functions from ListBoxModel,
         // telling the listbox how many rows there are, painting them, etc.
@@ -1323,10 +1370,10 @@ private:
 
     //==============================================================================
     // and this is a component that can have things dropped onto it..
-    class DragAndDropDemoTarget : public Component,
-                                  public DragAndDropTarget,
-                                  public FileDragAndDropTarget,
-                                  public TextDragAndDropTarget
+    class DragAndDropDemoTarget final : public Component,
+                                        public DragAndDropTarget,
+                                        public FileDragAndDropTarget,
+                                        public TextDragAndDropTarget
     {
     public:
         DragAndDropDemoTarget()    {}
@@ -1466,14 +1513,14 @@ private:
 };
 
 //==============================================================================
-struct DemoTabbedComponent  : public TabbedComponent
+struct DemoTabbedComponent final : public TabbedComponent
 {
-    DemoTabbedComponent (bool isRunningComponenTransformsDemo)
+    DemoTabbedComponent (bool isRunningComponentTransformsDemo)
         : TabbedComponent (TabbedButtonBar::TabsAtTop)
     {
         auto colour = findColour (ResizableWindow::backgroundColourId);
 
-        addTab ("Buttons",     colour, new ButtonsPage (isRunningComponenTransformsDemo), true);
+        addTab ("Buttons",     colour, new ButtonsPage (isRunningComponentTransformsDemo), true);
         addTab ("Sliders",     colour, new SlidersPage(),                                 true);
         addTab ("Toolbars",    colour, new ToolbarDemoComp(),                             true);
         addTab ("Misc",        colour, new MiscPage(),                                    true);
@@ -1481,17 +1528,17 @@ struct DemoTabbedComponent  : public TabbedComponent
         addTab ("Tables",      colour, new TableDemoComponent(),                          true);
         addTab ("Drag & Drop", colour, new DragAndDropDemo(),                             true);
 
-        getTabbedButtonBar().getTabButton (5)->setExtraComponent (new CustomTabButton (isRunningComponenTransformsDemo),
+        getTabbedButtonBar().getTabButton (5)->setExtraComponent (new CustomTabButton (isRunningComponentTransformsDemo),
                                                                   TabBarButton::afterText);
     }
 
     // This is a small star button that is put inside one of the tabs. You can
     // use this technique to create things like "close tab" buttons, etc.
-    class CustomTabButton  : public Component
+    class CustomTabButton final : public Component
     {
     public:
-        CustomTabButton (bool isRunningComponenTransformsDemo)
-            : runningComponenTransformsDemo (isRunningComponenTransformsDemo)
+        CustomTabButton (bool isRunningComponentTransformsDemo)
+            : runningComponentTransformsDemo (isRunningComponentTransformsDemo)
         {
             setSize (20, 20);
         }
@@ -1513,10 +1560,10 @@ struct DemoTabbedComponent  : public TabbedComponent
                                "You can use these to implement things like close-buttons "
                                "or status displays for your tabs.",
                                bubbleMessage,
-                               runningComponenTransformsDemo);
+                               runningComponentTransformsDemo);
         }
     private:
-        bool runningComponenTransformsDemo;
+        bool runningComponentTransformsDemo;
         std::unique_ptr<BubbleMessageComponent> bubbleMessage;
     };
 
@@ -1524,7 +1571,7 @@ struct DemoTabbedComponent  : public TabbedComponent
 };
 
 //==============================================================================
-struct WidgetsDemo   : public Component
+struct WidgetsDemo final : public Component
 {
     WidgetsDemo (bool isRunningComponenTransformsDemo = false)
         : tabs (isRunningComponenTransformsDemo)
