@@ -1,18 +1,22 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE examples.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework examples.
+   Copyright (c) Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
-   To use, copy, modify, and/or distribute this software for any purpose with or
+   to use, copy, modify, and/or distribute this software for any purpose with or
    without fee is hereby granted provided that the above copyright notice and
    this permission notice appear in all copies.
 
-   THE SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES,
-   WHETHER EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR
-   PURPOSE, ARE DISCLAIMED.
+   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+   REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+   AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
+   INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+   PERFORMANCE OF THIS SOFTWARE.
 
   ==============================================================================
 */
@@ -55,7 +59,7 @@
 
 //==============================================================================
 /** A demo synth sound that's just a basic sine wave.. */
-class SineWaveSound : public SynthesiserSound
+class SineWaveSound final : public SynthesiserSound
 {
 public:
     SineWaveSound() {}
@@ -66,7 +70,7 @@ public:
 
 //==============================================================================
 /** A simple demo synth voice that just plays a sine wave.. */
-class SineWaveVoice   : public SynthesiserVoice
+class SineWaveVoice final : public SynthesiserVoice
 {
 public:
     SineWaveVoice() {}
@@ -97,8 +101,8 @@ public:
             // start a tail-off by setting this flag. The render callback will pick up on
             // this and do a fade out, calling clearCurrentNote() when it's finished.
 
-            if (tailOff == 0.0) // we only need to begin a tail-off if it's not already doing so - the
-                                // stopNote method could be called more than once.
+            if (approximatelyEqual (tailOff, 0.0)) // we only need to begin a tail-off if it's not already doing so - the
+                                                   // stopNote method could be called more than once.
                 tailOff = 1.0;
         }
         else
@@ -122,7 +126,7 @@ public:
 
     void renderNextBlock (AudioBuffer<float>& outputBuffer, int startSample, int numSamples) override
     {
-        if (angleDelta != 0.0)
+        if (! approximatelyEqual (angleDelta, 0.0))
         {
             if (tailOff > 0.0)
             {
@@ -175,7 +179,7 @@ private:
 
 //==============================================================================
 /** As the name suggest, this class does the actual audio processing. */
-class JuceDemoPluginAudioProcessor  : public AudioProcessor
+class JuceDemoPluginAudioProcessor final : public AudioProcessor
 {
 public:
     //==============================================================================
@@ -362,9 +366,9 @@ public:
 private:
     //==============================================================================
     /** This is the editor component that our filter will display. */
-    class JuceDemoPluginAudioProcessorEditor  : public AudioProcessorEditor,
-                                                private Timer,
-                                                private Value::Listener
+    class JuceDemoPluginAudioProcessorEditor final : public AudioProcessorEditor,
+                                                     private Timer,
+                                                     private Value::Listener
     {
     public:
         JuceDemoPluginAudioProcessorEditor (JuceDemoPluginAudioProcessor& owner)
@@ -382,17 +386,17 @@ private:
 
             // add some labels for the sliders..
             gainLabel.attachToComponent (&gainSlider, false);
-            gainLabel.setFont (Font (11.0f));
+            gainLabel.setFont (FontOptions (11.0f));
 
             delayLabel.attachToComponent (&delaySlider, false);
-            delayLabel.setFont (Font (11.0f));
+            delayLabel.setFont (FontOptions (11.0f));
 
             // add the midi keyboard component..
             addAndMakeVisible (midiKeyboard);
 
             // add a label that will display the current timecode and status..
             addAndMakeVisible (timecodeDisplayLabel);
-            timecodeDisplayLabel.setFont (Font (Font::getDefaultMonospacedFontName(), 15.0f, Font::plain));
+            timecodeDisplayLabel.setFont (FontOptions (Font::getDefaultMonospacedFontName(), 15.0f, Font::plain));
 
             // set resize limits for this plug-in
             setResizeLimits (400, 200, 1024, 700);
