@@ -1,18 +1,22 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE examples.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework examples.
+   Copyright (c) Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
-   To use, copy, modify, and/or distribute this software for any purpose with or
+   to use, copy, modify, and/or distribute this software for any purpose with or
    without fee is hereby granted provided that the above copyright notice and
    this permission notice appear in all copies.
 
-   THE SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES,
-   WHETHER EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR
-   PURPOSE, ARE DISCLAIMED.
+   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+   REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+   AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
+   INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+   PERFORMANCE OF THIS SOFTWARE.
 
   ==============================================================================
 */
@@ -111,8 +115,8 @@ public class JuceActivity   extends Activity
     @Override
     protected void onNewIntent (Intent intent)
     {
-        super.onNewIntent(intent);
-        setIntent(intent);
+        super.onNewIntent (intent);
+        setIntent (intent);
 
         appNewIntent (intent);
     }
@@ -121,10 +125,10 @@ public class JuceActivity   extends Activity
 */
 
 //==============================================================================
-class PushNotificationsDemo   : public Component,
-                                private ChangeListener,
-                                private ComponentListener,
-                                private PushNotifications::Listener
+class PushNotificationsDemo final : public Component,
+                                    private ChangeListener,
+                                    private ComponentListener,
+                                    private PushNotifications::Listener
 {
 public:
     //==============================================================================
@@ -185,21 +189,25 @@ public:
             { PushNotifications::getInstance()->removeAllPendingLocalNotifications(); };
       #endif
 
-        remoteView.getDeviceTokenButton.onClick = []
+        remoteView.getDeviceTokenButton.onClick = [this]
         {
             String token = PushNotifications::getInstance()->getDeviceToken();
 
             DBG ("token = " + token);
 
             if (token.isEmpty())
+            {
                 showRemoteInstructions();
+            }
             else
-                NativeMessageBox::showAsync (MessageBoxOptions()
+            {
+                auto options = MessageBoxOptions()
                                                .withIconType (MessageBoxIconType::InfoIcon)
                                                .withTitle ("Device token")
                                                .withMessage (token)
-                                               .withButton ("OK"),
-                                             nullptr);
+                                               .withButton ("OK");
+                messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
+            }
         };
 
       #if JUCE_ANDROID
@@ -313,12 +321,12 @@ private:
             String requiredFields = "all required fields";
           #endif
 
-            NativeMessageBox::showAsync (MessageBoxOptions()
+            auto options = MessageBoxOptions()
                                            .withIconType (MessageBoxIconType::InfoIcon)
                                            .withTitle ("Incorrect notifications setup")
                                            .withMessage ("Please make sure that " + requiredFields + " are set.")
-                                           .withButton ("OK"),
-                                         nullptr);
+                                           .withButton ("OK");
+            messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
 
             return;
         }
@@ -565,14 +573,14 @@ private:
     {
         ignoreUnused (isLocalNotification);
 
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Received notification")
                                        .withMessage ("ID: " + n.identifier
                                                      + ", title: " + n.title
                                                      + ", body: " + n.body)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
     void handleNotificationAction (bool isLocalNotification,
@@ -582,7 +590,7 @@ private:
     {
         ignoreUnused (isLocalNotification);
 
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Received notification action")
                                        .withMessage ("ID: " + n.identifier
@@ -590,22 +598,22 @@ private:
                                                      + ", body: " + n.body
                                                      + ", action: " + actionIdentifier
                                                      + ", optionalResponse: " + optionalResponse)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
 
         PushNotifications::getInstance()->removeDeliveredNotification (n.identifier);
     }
 
     void localNotificationDismissedByUser (const PushNotifications::Notification& n) override
     {
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Notification dismissed by a user")
                                        .withMessage ("ID: " + n.identifier
                                                      + ", title: " + n.title
                                                      + ", body: " + n.body)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
     void deliveredNotificationsListReceived (const Array<PushNotifications::Notification>& notifs) override
@@ -615,12 +623,12 @@ private:
         for (auto& n : notifs)
             text << "(" << n.identifier << ", " << n.title << ", " << n.body << "), ";
 
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Received notification list")
                                        .withMessage (text)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
     void pendingLocalNotificationsListReceived (const Array<PushNotifications::Notification>& notifs) override
@@ -630,54 +638,54 @@ private:
         for (auto& n : notifs)
             text << "(" << n.identifier << ", " << n.title << ", " << n.body << "), ";
 
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Pending notification list")
                                        .withMessage (text)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
     void deviceTokenRefreshed (const String& token) override
     {
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Device token refreshed")
                                        .withMessage (token)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
   #if JUCE_ANDROID
     void remoteNotificationsDeleted() override
     {
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Remote notifications deleted")
                                        .withMessage ("Some of the pending messages were removed!")
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
     void upstreamMessageSent (const String& messageId) override
     {
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Upstream message sent")
                                        .withMessage ("Message id: " + messageId)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
     void upstreamMessageSendingError (const String& messageId, const String& error) override
     {
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Upstream message sending error")
                                        .withMessage ("Message id: " + messageId
                                                      + "\nerror: " + error)
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
     }
 
     static Array<PushNotifications::Channel> getAndroidChannels()
@@ -777,7 +785,7 @@ private:
     }
   #endif
 
-    struct RowComponent : public Component
+    struct RowComponent final : public Component
     {
         RowComponent (Label& l, Component& c, int u = 1)
             : label (l),
@@ -1083,7 +1091,7 @@ private:
       #endif
     }
 
-    struct ParamsView   : public Component
+    struct ParamsView final : public Component
     {
         ParamsView()
         {
@@ -1119,7 +1127,7 @@ private:
         OwnedArray<RowComponent> rowComponents;
     };
 
-    struct AuxActionsView   : public Component
+    struct AuxActionsView final : public Component
     {
         AuxActionsView()
         {
@@ -1173,7 +1181,7 @@ private:
         TextButton removeAllPendingNotifsButton     { "Remove All Pending Notifs" };
     };
 
-    struct RemoteView   : public Component
+    struct RemoteView final : public Component
     {
         RemoteView()
         {
@@ -1187,7 +1195,7 @@ private:
 
         void resized()
         {
-            auto rowSize = getHeight () / 10;
+            auto rowSize = getHeight() / 10;
 
             auto bounds = getLocalBounds().reduced (getWidth() / 10, getHeight() / 10);
 
@@ -1205,10 +1213,10 @@ private:
         TextButton unsubscribeFromSportsButton { "UnsubscribeFromSports" };
     };
 
-    struct DemoTabbedComponent  : public TabbedComponent
+    struct DemoTabbedComponent final : public TabbedComponent
     {
-        explicit DemoTabbedComponent (TabbedButtonBar::Orientation orientation)
-            : TabbedComponent (orientation)
+        DemoTabbedComponent (PushNotificationsDemo& demoIn, TabbedButtonBar::Orientation orientation)
+            : TabbedComponent (orientation), demo (demoIn)
         {
         }
 
@@ -1216,27 +1224,28 @@ private:
         {
             if (! showedRemoteInstructions && newCurrentTabName == "Remote")
             {
-                PushNotificationsDemo::showRemoteInstructions();
+                demo.showRemoteInstructions();
                 showedRemoteInstructions = true;
             }
         }
 
     private:
         bool showedRemoteInstructions = false;
+        PushNotificationsDemo& demo;
     };
 
-    static void showRemoteInstructions()
+    void showRemoteInstructions()
     {
        #if JUCE_IOS || JUCE_MAC
-        NativeMessageBox::showAsync (MessageBoxOptions()
+        auto options = MessageBoxOptions()
                                        .withIconType (MessageBoxIconType::InfoIcon)
                                        .withTitle ("Remote Notifications instructions")
                                        .withMessage ("In order to be able to test remote notifications "
                                                      "ensure that the app is signed and that you register "
                                                      "the bundle ID for remote notifications in "
                                                      "Apple Developer Center.")
-                                       .withButton ("OK"),
-                                     nullptr);
+                                       .withButton ("OK");
+        messageBox = NativeMessageBox::showScopedAsync (options, nullptr);
        #endif
     }
 
@@ -1246,10 +1255,11 @@ private:
     AuxActionsView auxActionsView;
     TabbedComponent localNotificationsTabs { TabbedButtonBar::TabsAtTop };
     RemoteView remoteView;
-    DemoTabbedComponent mainTabs { TabbedButtonBar::TabsAtTop };
+    DemoTabbedComponent mainTabs { *this, TabbedButtonBar::TabsAtTop };
     TextButton sendButton      { "Send!" };
     Label notAvailableYetLabel { "notAvailableYetLabel",
                                  "Push Notifications feature is not available on this platform yet!" };
+    ScopedMessageBox messageBox;
 
     //==============================================================================
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (PushNotificationsDemo)
