@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -33,8 +42,8 @@
  class AUScanner
  {
  public:
-     AUScanner (KnownPluginList& list)
-         : knownPluginList (list), pool (5)
+     explicit AUScanner (KnownPluginList& list)
+         : knownPluginList (list)
      {
          knownPluginList.clearBlacklistedFiles();
          paths = formatToScan.getDefaultLocationsToSearch();
@@ -49,7 +58,8 @@
      std::unique_ptr<PluginDirectoryScanner> scanner;
      FileSearchPath paths;
 
-     ThreadPool pool;
+     static constexpr auto numJobs = 5;
+     ThreadPool pool { ThreadPoolOptions{}.withNumberOfThreads (numJobs) };
 
      void startScan()
      {
@@ -59,24 +69,21 @@
          scanner.reset (new PluginDirectoryScanner (knownPluginList, formatToScan, paths,
                                                     true, deadMansPedalFile, true));
 
-         for (int i = 5; --i >= 0;)
+         for (int i = numJobs; --i >= 0;)
              pool.addJob (new ScanJob (*this), true);
      }
 
      bool doNextScan()
      {
          String pluginBeingScanned;
-         if (scanner->scanNextFile (true, pluginBeingScanned))
-             return true;
-
-         return false;
+         return scanner->scanNextFile (true, pluginBeingScanned);
      }
 
-     struct ScanJob  : public ThreadPoolJob
+     struct ScanJob final : public ThreadPoolJob
      {
          ScanJob (AUScanner& s)  : ThreadPoolJob ("pluginscan"), scanner (s) {}
 
-         JobStatus runJob()
+         JobStatus runJob() override
          {
              while (scanner.doNextScan() && ! shouldExit())
              {}
@@ -94,8 +101,8 @@
 #endif
 
 //==============================================================================
-struct GraphEditorPanel::PinComponent   : public Component,
-                                          public SettableTooltipClient
+struct GraphEditorPanel::PinComponent final : public Component,
+                                              public SettableTooltipClient
 {
     PinComponent (GraphEditorPanel& p, AudioProcessorGraph::NodeAndChannel pinToUse, bool isIn)
         : panel (p), graph (p.graph), pin (pinToUse), isInput (isIn)
@@ -172,10 +179,10 @@ struct GraphEditorPanel::PinComponent   : public Component,
 };
 
 //==============================================================================
-struct GraphEditorPanel::PluginComponent   : public Component,
-                                             public Timer,
-                                             private AudioProcessorParameter::Listener,
-                                             private AsyncUpdater
+struct GraphEditorPanel::PluginComponent final : public Component,
+                                                 public Timer,
+                                                 private AudioProcessorParameter::Listener,
+                                                 private AsyncUpdater
 {
     PluginComponent (GraphEditorPanel& p, AudioProcessorGraph::NodeID id)  : panel (p), graph (p.graph), pluginID (id)
     {
@@ -546,7 +553,7 @@ struct GraphEditorPanel::PluginComponent   : public Component,
     int numInputs = 0, numOutputs = 0;
     int pinSize = 16;
     Point<int> originalPos;
-    Font font { 13.0f, Font::bold };
+    Font font = FontOptions { 13.0f, Font::bold };
     int numIns = 0, numOuts = 0;
     DropShadowEffect shadow;
     std::unique_ptr<PopupMenu> menu;
@@ -556,8 +563,8 @@ struct GraphEditorPanel::PluginComponent   : public Component,
 
 
 //==============================================================================
-struct GraphEditorPanel::ConnectorComponent   : public Component,
-                                                public SettableTooltipClient
+struct GraphEditorPanel::ConnectorComponent final : public Component,
+                                                    public SettableTooltipClient
 {
     explicit ConnectorComponent (GraphEditorPanel& p)
         : panel (p), graph (p.graph)
@@ -849,11 +856,11 @@ void GraphEditorPanel::changeListenerCallback (ChangeBroadcaster*)
 void GraphEditorPanel::updateComponents()
 {
     for (int i = nodes.size(); --i >= 0;)
-        if (graph.graph.getNodeForId (nodes.getUnchecked(i)->pluginID) == nullptr)
+        if (graph.graph.getNodeForId (nodes.getUnchecked (i)->pluginID) == nullptr)
             nodes.remove (i);
 
     for (int i = connectors.size(); --i >= 0;)
-        if (! graph.graph.isConnected (connectors.getUnchecked(i)->connection))
+        if (! graph.graph.isConnected (connectors.getUnchecked (i)->connection))
             connectors.remove (i);
 
     for (auto* fc : nodes)
@@ -896,9 +903,9 @@ void GraphEditorPanel::showPopupMenu (Point<int> mousePos)
         menu->showMenuAsync ({},
                              ModalCallbackFunction::create ([this, mousePos] (int r)
                                                             {
-                                                                if (r > 0)
-                                                                    if (auto* mainWin = findParentComponentOfClass<MainHostWindow>())
-                                                                        createNewPlugin (mainWin->getChosenType (r), mousePos);
+                                                                if (auto* mainWin = findParentComponentOfClass<MainHostWindow>())
+                                                                    if (const auto chosen = mainWin->getChosenType (r))
+                                                                        createNewPlugin (*chosen, mousePos);
                                                             }));
     }
 }
@@ -1003,8 +1010,8 @@ void GraphEditorPanel::timerCallback()
 }
 
 //==============================================================================
-struct GraphDocumentComponent::TooltipBar   : public Component,
-                                              private Timer
+struct GraphDocumentComponent::TooltipBar final : public Component,
+                                                  private Timer
 {
     TooltipBar()
     {
@@ -1013,7 +1020,7 @@ struct GraphDocumentComponent::TooltipBar   : public Component,
 
     void paint (Graphics& g) override
     {
-        g.setFont (Font ((float) getHeight() * 0.7f, Font::bold));
+        g.setFont (FontOptions ((float) getHeight() * 0.7f, Font::bold));
         g.setColour (Colours::black);
         g.drawFittedText (tip, 10, 0, getWidth() - 12, getHeight(), Justification::centredLeft, 1);
     }
@@ -1040,8 +1047,8 @@ struct GraphDocumentComponent::TooltipBar   : public Component,
 };
 
 //==============================================================================
-class GraphDocumentComponent::TitleBarComponent    : public Component,
-                                                     private Button::Listener
+class GraphDocumentComponent::TitleBarComponent final : public Component,
+                                                        private Button::Listener
 {
 public:
     explicit TitleBarComponent (GraphDocumentComponent& graphDocumentComponent)
@@ -1117,7 +1124,7 @@ private:
 
         pluginButton.setBounds (r.removeFromRight (40).withSizeKeepingCentre (20, 20));
 
-        titleLabel.setFont (Font (static_cast<float> (getHeight()) * 0.5f, Font::plain));
+        titleLabel.setFont (FontOptions (static_cast<float> (getHeight()) * 0.5f, Font::plain));
         titleLabel.setBounds (r);
     }
 
@@ -1136,9 +1143,9 @@ private:
 };
 
 //==============================================================================
-struct GraphDocumentComponent::PluginListBoxModel    : public ListBoxModel,
-                                                       public ChangeListener,
-                                                       public MouseListener
+struct GraphDocumentComponent::PluginListBoxModel final : public ListBoxModel,
+                                                          public ChangeListener,
+                                                          public MouseListener
 {
     PluginListBoxModel (ListBox& lb, KnownPluginList& kpl)
         : owner (lb),
@@ -1285,7 +1292,7 @@ void GraphDocumentComponent::resized()
     const int statusHeight = 20;
 
     if (isOnTouchDevice())
-        titleBarComponent->setBounds (r.removeFromTop(titleBarHeight));
+        titleBarComponent->setBounds (r.removeFromTop (titleBarHeight));
 
     keyboardComp->setBounds (r.removeFromBottom (keysHeight));
     statusBar->setBounds (r.removeFromBottom (statusHeight));
