@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -56,11 +65,14 @@ void LatestVersionCheckerAndUpdater::run()
     if (info == nullptr)
     {
         if (! backgroundCheck)
-            AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
-                                              "Update Server Communication Error",
-                                              "Failed to communicate with the JUCE update server.\n"
-                                              "Please try again in a few minutes.\n\n"
-                                              "If this problem persists you can download the latest version of JUCE from juce.com");
+        {
+            auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::WarningIcon,
+                                                             "Update Server Communication Error",
+                                                             "Failed to communicate with the JUCE update server.\n"
+                                                             "Please try again in a few minutes.\n\n"
+                                                             "If this problem persists you can download the latest version of JUCE from juce.com");
+            messageBox = AlertWindow::showScopedAsync (options, nullptr);
+        }
 
         return;
     }
@@ -68,9 +80,12 @@ void LatestVersionCheckerAndUpdater::run()
     if (! info->isNewerVersionThanCurrent())
     {
         if (! backgroundCheck)
-            AlertWindow::showMessageBoxAsync (MessageBoxIconType::InfoIcon,
-                                              "No New Version Available",
-                                              "Your JUCE version is up to date.");
+        {
+            auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::InfoIcon,
+                                                             "No New Version Available",
+                                                             "Your JUCE version is up to date.");
+            messageBox = AlertWindow::showScopedAsync (options, nullptr);
+        }
         return;
     }
 
@@ -109,29 +124,32 @@ void LatestVersionCheckerAndUpdater::run()
     }
 
     if (! backgroundCheck)
-        AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
-                                          "Failed to find any new downloads",
-                                          "Please try again in a few minutes.");
+    {
+        auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::WarningIcon,
+                                                         "Failed to find any new downloads",
+                                                         "Please try again in a few minutes.");
+        messageBox = AlertWindow::showScopedAsync (options, nullptr);
+    }
 }
 
 //==============================================================================
-class UpdateDialog  : public Component
+class UpdateDialog final : public Component
 {
 public:
     UpdateDialog (const String& newVersion, const String& releaseNotes)
     {
         titleLabel.setText ("JUCE version " + newVersion, dontSendNotification);
-        titleLabel.setFont ({ 15.0f, Font::bold });
+        titleLabel.setFont (FontOptions { 15.0f, Font::bold });
         titleLabel.setJustificationType (Justification::centred);
         addAndMakeVisible (titleLabel);
 
         contentLabel.setText ("A new version of JUCE is available - would you like to download it?", dontSendNotification);
-        contentLabel.setFont (15.0f);
+        contentLabel.setFont (FontOptions { 15.0f });
         contentLabel.setJustificationType (Justification::topLeft);
         addAndMakeVisible (contentLabel);
 
         releaseNotesLabel.setText ("Release notes:", dontSendNotification);
-        releaseNotesLabel.setFont (15.0f);
+        releaseNotesLabel.setFont (FontOptions { 15.0f });
         releaseNotesLabel.setJustificationType (Justification::topLeft);
         addAndMakeVisible (releaseNotesLabel);
 
@@ -155,7 +173,7 @@ public:
 
         juceIcon = Drawable::createFromImageData (BinaryData::juce_icon_png,
                                                   BinaryData::juce_icon_pngSize);
-        lookAndFeelChanged();
+        updateLookAndFeel();
 
         setSize (500, 280);
     }
@@ -209,12 +227,17 @@ public:
     }
 
 private:
-    void lookAndFeelChanged() override
+    void updateLookAndFeel()
     {
         cancelButton.setColour (TextButton::buttonColourId, findColour (secondaryButtonBackgroundColourId));
         releaseNotesEditor.applyFontToAllText (releaseNotesEditor.getFont());
     }
 
+    void lookAndFeelChanged() override
+    {
+        updateLookAndFeel();
+    }
+
     void setParentWindow (DialogWindow* parent)
     {
         parentWindow = parent;
@@ -275,33 +298,38 @@ void LatestVersionCheckerAndUpdater::askUserForLocationToDownload (const Version
         {
             if (targetFolder.getChildFile (".git").isDirectory())
             {
-                AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon, "Downloading New JUCE Version",
-                                                  targetFolderPath + "\n\nis a GIT repository!\n\nYou should use a \"git pull\" to update it to the latest version.");
+                auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::WarningIcon,
+                                                                 "Downloading New JUCE Version",
+                                                                 targetFolderPath + "\n\n"
+                                                                 "is a GIT repository!\n\n"
+                                                                 "You should use a \"git pull\" to update it to the latest version.");
+                if (weakThis != nullptr)
+                    weakThis->messageBox = AlertWindow::showScopedAsync (options, nullptr);
 
                 return;
             }
 
-            AlertWindow::showOkCancelBox (MessageBoxIconType::WarningIcon,
-                                          "Overwrite Existing JUCE Folder?",
-                                          "Do you want to replace the folder\n\n" + targetFolderPath + "\n\nwith the latest version from juce.com?\n\n"
-                                              "This will move the existing folder to " + targetFolderPath + "_old.\n\n"
-                                              "Replacing the folder that contains the currently running Projucer executable may not work on Windows.",
-                                          {},
-                                          {},
-                                          nullptr,
-                                          ModalCallbackFunction::create (onResult));
+            auto options = MessageBoxOptions::makeOptionsOkCancel (MessageBoxIconType::WarningIcon,
+                                                                   "Overwrite Existing JUCE Folder?",
+                                                                   "Do you want to replace the folder\n\n" + targetFolderPath + "\n\n"
+                                                                   "with the latest version from juce.com?\n\n"
+                                                                   "This will move the existing folder to " + targetFolderPath + "_old.\n\n"
+                                                                   "Replacing the folder that contains the currently running Projucer executable may not work on Windows.");
+            if (weakThis != nullptr)
+                weakThis->messageBox = AlertWindow::showScopedAsync (options, onResult);
+
             return;
         }
 
         if (targetFolder.exists())
         {
-            AlertWindow::showOkCancelBox (MessageBoxIconType::WarningIcon,
-                                          "Existing File Or Directory",
-                                          "Do you want to move\n\n" + targetFolderPath + "\n\nto\n\n" + targetFolderPath + "_old?",
-                                          {},
-                                          {},
-                                          nullptr,
-                                          ModalCallbackFunction::create (onResult));
+            auto options = MessageBoxOptions::makeOptionsOkCancel (MessageBoxIconType::WarningIcon,
+                                                                   "Existing File Or Directory",
+                                                                   "Do you want to move\n\n" + targetFolderPath + "\n\n"
+                                                                   "to\n\n" + targetFolderPath + "_old?");
+            if (weakThis != nullptr)
+                weakThis->messageBox = AlertWindow::showScopedAsync (options, onResult);
+
             return;
         }
 
@@ -366,10 +394,10 @@ void LatestVersionCheckerAndUpdater::addNotificationToOpenProjects (const Versio
 }
 
 //==============================================================================
-class DownloadAndInstallThread   : private ThreadWithProgressWindow
+class DownloadAndInstallThread final : private ThreadWithProgressWindow
 {
 public:
-    DownloadAndInstallThread  (const VersionInfo::Asset& a, const File& t, std::function<void()>&& cb)
+    DownloadAndInstallThread  (const VersionInfo::Asset& a, const File& t, std::function<void (Result)>&& cb)
         : ThreadWithProgressWindow ("Downloading New Version", true, true),
           asset (a), targetFolder (t), completionCallback (std::move (cb))
     {
@@ -387,12 +415,10 @@ private:
         if (result.wasOk() && ! threadShouldExit())
             result = install (zipData);
 
-        if (result.failed())
-            MessageManager::callAsync ([result] { AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
-                                                                                    "Installation Failed",
-                                                                                    result.getErrorMessage()); });
-        else
-            MessageManager::callAsync (completionCallback);
+        MessageManager::callAsync ([result, callback = completionCallback]
+        {
+            callback (result);
+        });
     }
 
     Result download (MemoryBlock& dest)
@@ -517,7 +543,7 @@ private:
 
     VersionInfo::Asset asset;
     File targetFolder;
-    std::function<void()> completionCallback;
+    std::function<void (Result)> completionCallback;
 };
 
 static void restartProcess (const File& targetFolder)
@@ -548,12 +574,22 @@ static void restartProcess (const File& targetFolder)
 
 void LatestVersionCheckerAndUpdater::downloadAndInstall (const VersionInfo::Asset& asset, const File& targetFolder)
 {
-    installer.reset (new DownloadAndInstallThread (asset, targetFolder,
-                                                   [this, targetFolder]
-                                                   {
-                                                       installer.reset();
-                                                       restartProcess (targetFolder);
-                                                   }));
+    installer.reset (new DownloadAndInstallThread (asset, targetFolder, [this, targetFolder] (const auto result)
+    {
+        if (result.failed())
+        {
+            auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::WarningIcon,
+                                                             "Installation Failed",
+                                                             result.getErrorMessage());
+
+            messageBox = AlertWindow::showScopedAsync (options, nullptr);
+        }
+        else
+        {
+            installer.reset();
+            restartProcess (targetFolder);
+        }
+    }));
 }
 
 //==============================================================================
