@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -179,12 +188,12 @@ public:
         {
         }
 
-        void setResource (const String& newName)
+        void setResource (const String& newName) override
         {
             setImageResource (layout, element, role, newName, true);
         }
 
-        String getResource() const
+        String getResource() const override
         {
             return getImageResource (element, role);
         }
@@ -294,7 +303,7 @@ public:
     {
         if (undoable)
         {
-            layout.perform (new SetImageKeepsPropAction (button, layout, newState), "change imagebutton proportion mode");
+            layout.perform (std::make_unique<SetImageKeepsPropAction> (button, layout, newState), "change imagebutton proportion mode");
         }
         else
         {
@@ -314,12 +323,12 @@ public:
         {
         }
 
-        void setState (bool newState)
+        void setState (bool newState) override
         {
             setImageKeepProportions (layout, component, newState, true);
         }
 
-        bool getState() const
+        bool getState() const override
         {
             return doesImageKeepProportions (component);
         }
@@ -371,7 +380,7 @@ public:
     {
         if (undoable)
         {
-            layout.perform (new SetImageOpacityAction (button, layout, role, opacity), "change imagebutton opacity");
+            layout.perform (std::make_unique<SetImageOpacityAction> (button, layout, role, opacity), "change imagebutton opacity");
         }
         else
         {
@@ -453,7 +462,7 @@ public:
     {
         if (undoable)
         {
-            layout.perform (new SetImageColourAction (button, layout, role, colour), "change imagebutton colour");
+            layout.perform (std::make_unique<SetImageColourAction> (button, layout, role, colour), "change imagebutton colour");
         }
         else
         {
