@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -143,13 +152,13 @@ private:
             choices.add ("multi-line, return key disabled");
         }
 
-        void setIndex (int newIndex)
+        void setIndex (int newIndex) override
         {
             document.perform (new TextEditorMultilineChangeAction (component, *document.getComponentLayout(), newIndex),
                               "Change TextEditor multiline mode");
         }
 
-        int getIndex() const
+        int getIndex() const override
         {
             return component->isMultiLine() ? (component->getReturnKeyStartsNewLine() ? 1 : 2) : 0;
         }
@@ -165,7 +174,7 @@ private:
                 oldState = comp->isMultiLine() ? (comp->getReturnKeyStartsNewLine() ? 1 : 2) : 0;
             }
 
-            bool perform()
+            bool perform() override
             {
                 showCorrectTab();
                 getComponent()->setMultiLine (newState > 0);
@@ -174,7 +183,7 @@ private:
                 return true;
             }
 
-            bool undo()
+            bool undo() override
             {
                 showCorrectTab();
                 getComponent()->setMultiLine (oldState > 0);
@@ -196,13 +205,13 @@ private:
         {
         }
 
-        void setState (bool newState)
+        void setState (bool newState) override
         {
             document.perform (new TextEditorReadonlyChangeAction (component, *document.getComponentLayout(), ! newState),
                               "Change TextEditor read-only mode");
         }
 
-        bool getState() const        { return ! component->isReadOnly(); }
+        bool getState() const override        { return ! component->isReadOnly(); }
 
     private:
         class TextEditorReadonlyChangeAction  : public ComponentUndoableAction <TextEditor>
@@ -215,7 +224,7 @@ private:
                 oldState = comp->isReadOnly();
             }
 
-            bool perform()
+            bool perform() override
             {
                 showCorrectTab();
                 getComponent()->setReadOnly (newState);
@@ -223,7 +232,7 @@ private:
                 return true;
             }
 
-            bool undo()
+            bool undo() override
             {
                 showCorrectTab();
                 getComponent()->setReadOnly (oldState);
@@ -244,13 +253,13 @@ private:
         {
         }
 
-        void setState (bool newState)
+        void setState (bool newState) override
         {
             document.perform (new TextEditorScrollbarChangeAction (component, *document.getComponentLayout(), newState),
                               "Change TextEditor scrollbars");
         }
 
-        bool getState() const        { return component->areScrollbarsShown(); }
+        bool getState() const override        { return component->areScrollbarsShown(); }
 
     private:
         class TextEditorScrollbarChangeAction  : public ComponentUndoableAction <TextEditor>
@@ -263,7 +272,7 @@ private:
                 oldState = comp->areScrollbarsShown();
             }
 
-            bool perform()
+            bool perform() override
             {
                 showCorrectTab();
                 getComponent()->setScrollbarsShown (newState);
@@ -271,7 +280,7 @@ private:
                 return true;
             }
 
-            bool undo()
+            bool undo() override
             {
                 showCorrectTab();
                 getComponent()->setScrollbarsShown (oldState);
@@ -292,13 +301,13 @@ private:
         {
         }
 
-        void setState (bool newState)
+        void setState (bool newState) override
         {
             document.perform (new TextEditorCaretChangeAction (component, *document.getComponentLayout(), newState),
                               "Change TextEditor caret");
         }
 
-        bool getState() const        { return component->isCaretVisible(); }
+        bool getState() const override        { return component->isCaretVisible(); }
 
     private:
         class TextEditorCaretChangeAction  : public ComponentUndoableAction <TextEditor>
@@ -311,7 +320,7 @@ private:
                 oldState = comp->isCaretVisible();
             }
 
-            bool perform()
+            bool perform() override
             {
                 showCorrectTab();
                 getComponent()->setCaretVisible (newState);
@@ -319,7 +328,7 @@ private:
                 return true;
             }
 
-            bool undo()
+            bool undo() override
             {
                 showCorrectTab();
                 getComponent()->setCaretVisible (oldState);
@@ -340,13 +349,13 @@ private:
         {
         }
 
-        void setState (bool newState)
+        void setState (bool newState) override
         {
             document.perform (new TextEditorPopupMenuChangeAction (component, *document.getComponentLayout(), newState),
                               "Change TextEditor popup menu");
         }
 
-        bool getState() const        { return component->isPopupMenuEnabled(); }
+        bool getState() const override        { return component->isPopupMenuEnabled(); }
 
     private:
         class TextEditorPopupMenuChangeAction  : public ComponentUndoableAction <TextEditor>
@@ -359,7 +368,7 @@ private:
                 oldState = comp->isPopupMenuEnabled();
             }
 
-            bool perform()
+            bool perform() override
             {
                 showCorrectTab();
                 getComponent()->setPopupMenuEnabled (newState);
@@ -367,7 +376,7 @@ private:
                 return true;
             }
 
-            bool undo()
+            bool undo() override
             {
                 showCorrectTab();
                 getComponent()->setPopupMenuEnabled (oldState);
@@ -409,7 +418,7 @@ private:
                 oldState = comp->getProperties() ["initialText"];
             }
 
-            bool perform()
+            bool perform() override
             {
                 showCorrectTab();
                 getComponent()->setText (newState, false);
@@ -418,7 +427,7 @@ private:
                 return true;
             }
 
-            bool undo()
+            bool undo() override
             {
                 showCorrectTab();
                 getComponent()->setText (oldState, false);
