@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -49,7 +58,7 @@ public:
         choices.add ("Image Brush");
     }
 
-    void setIndex (int newIndex)
+    void setIndex (int newIndex) override
     {
         JucerFillType fill (isForStroke ? listener.owner->getStrokeType().fill
                                         : listener.owner->getFillType());
@@ -69,7 +78,7 @@ public:
             listener.owner->setStrokeFill (fill, true);
     }
 
-    int getIndex() const
+    int getIndex() const override
     {
         switch (isForStroke ? listener.owner->getStrokeType().fill.mode
                             : listener.owner->getFillType().mode)
@@ -167,8 +176,8 @@ public:
     ElementFillPositionProperty (ColouredElement* const owner_,
                                  const String& name,
                                  ComponentPositionDimension dimension_,
-                                 const bool isStart_,
-                                 const bool isForStroke_)
+                                 bool isStart_,
+                                 bool isForStroke_)
      : PositionPropertyBase (owner_, name, dimension_, false, false,
                              owner_->getDocument()->getComponentLayout()),
        listener (owner_),
@@ -178,7 +187,7 @@ public:
         listener.setPropertyToRefresh (*this);
     }
 
-    void setPosition (const RelativePositionedRectangle& newPos)
+    void setPosition (const RelativePositionedRectangle& newPos) override
     {
         JucerFillType fill (isForStroke ? listener.owner->getStrokeType().fill
                                         : listener.owner->getFillType());
@@ -194,7 +203,7 @@ public:
             listener.owner->setStrokeFill (fill, true);
     }
 
-    RelativePositionedRectangle getPosition() const
+    RelativePositionedRectangle getPosition() const override
     {
         const JucerFillType fill (isForStroke ? listener.owner->getStrokeType().fill
                                               : listener.owner->getFillType());
@@ -220,8 +229,8 @@ public:
     }
 
     //==============================================================================
-    void setState (bool newState)           { listener.owner->enableStroke (newState, true); }
-    bool getState() const                   { return listener.owner->isStrokeEnabled(); }
+    void setState (bool newState) override           { listener.owner->enableStroke (newState, true); }
+    bool getState() const override                   { return listener.owner->isStrokeEnabled(); }
 
     ElementListener<ColouredElement> listener;
 };
@@ -237,7 +246,7 @@ public:
         listener.setPropertyToRefresh (*this);
     }
 
-    void setValue (double newValue)
+    void setValue (double newValue) override
     {
         listener.owner->getDocument()->getUndoManager().undoCurrentTransactionOnly();
 
@@ -247,7 +256,7 @@ public:
                                        true);
     }
 
-    double getValue() const                 { return listener.owner->getStrokeType().stroke.getStrokeThickness(); }
+    double getValue() const override                 { return listener.owner->getStrokeType().stroke.getStrokeThickness(); }
 
     ElementListener<ColouredElement> listener;
 };
@@ -267,7 +276,7 @@ public:
         choices.add ("beveled");
     }
 
-    void setIndex (int newIndex)
+    void setIndex (int newIndex) override
     {
         const PathStrokeType::JointStyle joints[] = { PathStrokeType::mitered,
                                                       PathStrokeType::curved,
@@ -285,7 +294,7 @@ public:
                                        true);
     }
 
-    int getIndex() const
+    int getIndex() const override
     {
         switch (listener.owner->getStrokeType().stroke.getJointStyle())
         {
@@ -316,7 +325,7 @@ public:
         choices.add ("round");
     }
 
-    void setIndex (int newIndex)
+    void setIndex (int newIndex) override
     {
         const PathStrokeType::EndCapStyle ends[] = { PathStrokeType::butt,
                                                      PathStrokeType::square,
@@ -334,7 +343,7 @@ public:
                                        true);
     }
 
-    int getIndex() const
+    int getIndex() const override
     {
         switch (listener.owner->getStrokeType().stroke.getEndStyle())
         {
@@ -362,7 +371,7 @@ public:
     }
 
     //==============================================================================
-    void setResource (const String& newName)
+    void setResource (const String& newName) override
     {
         if (element != nullptr)
         {
@@ -383,7 +392,7 @@ public:
         }
     }
 
-    String getResource() const
+    String getResource() const override
     {
         if (element == nullptr)
             return {};
@@ -414,7 +423,7 @@ public:
         listener.setPropertyToRefresh (*this);
     }
 
-    void setPosition (const RelativePositionedRectangle& newPos)
+    void setPosition (const RelativePositionedRectangle& newPos) override
     {
         if (isForStroke)
         {
@@ -430,7 +439,7 @@ public:
         }
     }
 
-    RelativePositionedRectangle getPosition() const
+    RelativePositionedRectangle getPosition() const override
     {
         if (isForStroke)
             return listener.owner->getStrokeType().fill.imageAnchor;
@@ -455,7 +464,7 @@ public:
         listener.setPropertyToRefresh (*this);
     }
 
-    void setValue (double newValue)
+    void setValue (double newValue) override
     {
         if (listener.owner != nullptr)
         {
@@ -478,7 +487,7 @@ public:
         }
     }
 
-    double getValue() const
+    double getValue() const override
     {
         if (listener.owner == nullptr)
             return 0;
@@ -616,14 +625,14 @@ public:
         oldState = element->getFillType();
     }
 
-    bool perform()
+    bool perform() override
     {
         showCorrectTab();
         getElement()->setFillType (newState, false);
         return true;
     }
 
-    bool undo()
+    bool undo() override
     {
         showCorrectTab();
         getElement()->setFillType (oldState, false);
@@ -675,14 +684,14 @@ public:
         oldState = element->isStrokeEnabled();
     }
 
-    bool perform()
+    bool perform() override
     {
         showCorrectTab();
         getElement()->enableStroke (newState, false);
         return true;
     }
 
-    bool undo()
+    bool undo() override
     {
         showCorrectTab();
         getElement()->enableStroke (oldState, false);
@@ -732,14 +741,14 @@ public:
     {
     }
 
-    bool perform()
+    bool perform() override
     {
         showCorrectTab();
         getElement()->setStrokeType (newState, false);
         return true;
     }
 
-    bool undo()
+    bool undo() override
     {
         showCorrectTab();
         getElement()->setStrokeType (oldState, false);
@@ -778,14 +787,14 @@ public:
         oldState = element->getStrokeType().fill;
     }
 
-    bool perform()
+    bool perform() override
     {
         showCorrectTab();
         getElement()->setStrokeFill (newState, false);
         return true;
     }
 
-    bool undo()
+    bool undo() override
     {
         showCorrectTab();
         getElement()->setStrokeFill (oldState, false);
