@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -36,7 +45,7 @@ public:
     PaintElementText (PaintRoutine* pr)
         : ColouredElement (pr, "Text", false, false),
           text ("Your text goes here"),
-          font (15.0f),
+          font (FontOptions { 15.0f }),
           typefaceName (FontPropertyComponent::getDefaultFont()),
           justification (Justification::centred)
     {
@@ -176,14 +185,14 @@ public:
         {
         }
 
-        bool perform()
+        bool perform() override
         {
             showCorrectTab();
             getElement()->setText (newText, false);
             return true;
         }
 
-        bool undo()
+        bool undo() override
         {
             showCorrectTab();
             getElement()->setText (oldText, false);
@@ -224,14 +233,14 @@ public:
         {
         }
 
-        bool perform()
+        bool perform() override
         {
             showCorrectTab();
             getElement()->setFont (newFont, false);
             return true;
         }
 
-        bool undo()
+        bool undo() override
         {
             showCorrectTab();
             getElement()->setFont (oldFont, false);
@@ -270,14 +279,14 @@ public:
         {
         }
 
-        bool perform()
+        bool perform() override
         {
             showCorrectTab();
             getElement()->setTypefaceName (newValue, false);
             return true;
         }
 
-        bool undo()
+        bool undo() override
         {
             showCorrectTab();
             getElement()->setTypefaceName (oldValue, false);
@@ -317,14 +326,14 @@ public:
         {
         }
 
-        bool perform()
+        bool perform() override
         {
             showCorrectTab();
             getElement()->setJustification (newValue, false);
             return true;
         }
 
-        bool undo()
+        bool undo() override
         {
             showCorrectTab();
             getElement()->setJustification (oldValue, false);
@@ -430,16 +439,16 @@ private:
             element->getDocument()->addChangeListener (this);
         }
 
-        ~FontNameProperty()
+        ~FontNameProperty() override
         {
             element->getDocument()->removeChangeListener (this);
         }
 
-        void setTypefaceName (const String& newFontName)    { element->setTypefaceName (newFontName, true); }
-        String getTypefaceName() const                      { return element->getTypefaceName(); }
+        void setTypefaceName (const String& newFontName) override    { element->setTypefaceName (newFontName, true); }
+        String getTypefaceName() const override                      { return element->getTypefaceName(); }
 
     private:
-        void changeListenerCallback (ChangeBroadcaster*)                 { refresh(); }
+        void changeListenerCallback (ChangeBroadcaster*) override                 { refresh(); }
 
         PaintElementText* const element;
     };
@@ -458,7 +467,7 @@ private:
             updateStylesList (element->getTypefaceName());
         }
 
-        ~FontStyleProperty()
+        ~FontStyleProperty() override
         {
             element->getDocument()->removeChangeListener (this);
         }
@@ -485,7 +494,7 @@ private:
             refresh();
         }
 
-        void setIndex (int newIndex)
+        void setIndex (int newIndex) override
         {
             Font f (element->getFont());
 
@@ -505,7 +514,7 @@ private:
             element->setFont (f, true);
         }
 
-        int getIndex() const
+        int getIndex() const override
         {
             auto f = element->getFont();
 
@@ -526,7 +535,7 @@ private:
         }
 
     private:
-        void changeListenerCallback (ChangeBroadcaster*)
+        void changeListenerCallback (ChangeBroadcaster*) override
         {
             updateStylesList (element->getTypefaceName());
         }
@@ -656,12 +665,12 @@ private:
         {
         }
 
-        void buttonClicked()
+        void buttonClicked() override
         {
             element->convertToPath();
         }
 
-        String getButtonText() const
+        String getButtonText() const override
         {
             return "convert text to a path";
         }
