@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -175,7 +184,7 @@ public:
         }
     }
 
-    void changeListenerCallback (ChangeBroadcaster*)
+    void changeListenerCallback (ChangeBroadcaster*) override
     {
         refresh();
     }
@@ -392,7 +401,7 @@ public:
         });
     }
 
-    void resized()
+    void resized() override
     {
         const Rectangle<int> r (getLookAndFeel().getPropertyComponentContentPosition (*this));
 
@@ -402,7 +411,7 @@ public:
         textEditor->setBounds (r.getX(), r.getY(), button.getX() - r.getX(), r.getHeight());
     }
 
-    void refresh()
+    void refresh() override
     {
         textEditor->setText (getText(), dontSendNotification);
     }
@@ -422,15 +431,13 @@ public:
 protected:
     class PositionPropLabel  : public Label
     {
-        PositionPropertyBase& owner;
-
     public:
         PositionPropLabel (PositionPropertyBase& owner_)
             : Label (String(), String()),
               owner (owner_)
         {
             setEditable (true, true, false);
-            lookAndFeelChanged();
+            updateLookAndFeel();
         }
 
         TextEditor* createEditorComponent() override
@@ -447,10 +454,18 @@ protected:
         }
 
         void lookAndFeelChanged() override
+        {
+            updateLookAndFeel();
+        }
+
+    private:
+        void updateLookAndFeel()
         {
             setColour (backgroundColourId, findColour (widgetBackgroundColourId));
             setColour (textColourId, findColour (widgetTextColourId));
         }
+
+        PositionPropertyBase& owner;
     };
 
     ComponentLayout* layout;
