@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -220,8 +229,8 @@ public:
 
     /** Same functionality as updateFrom(), but taking doubles instead of ints.
     */
-    void updateFromDouble (const double newX, const double newY,
-                           const double newW, const double newH,
+    void updateFromDouble (double newX, double newY,
+                           double newW, double newH,
                            const Rectangle<int>& target) noexcept
     {
         updatePosAndSize (x, w, newX, newW, xMode, wMode, target.getX(), target.getWidth());
@@ -334,7 +343,7 @@ public:
     /** Sets the raw value of the x coordinate.
         See getX() for the meaning of this value.
     */
-    void setX (const double newX) noexcept          { x = newX; }
+    void setX (double newX) noexcept                { x = newX; }
 
     /** Returns the anchoring mode for the y coordinate.
         To change any of the modes, use setModes().
@@ -364,7 +373,7 @@ public:
     /** Sets the raw value of the y coordinate.
         See getY() for the meaning of this value.
     */
-    void setY (const double newY) noexcept          { y = newY; }
+    void setY (double newY) noexcept          { y = newY; }
 
     /** Returns the mode used to calculate the width.
         To change any of the modes, use setModes().
@@ -383,7 +392,7 @@ public:
 
         See getWidth() for the details about what this value means.
     */
-    void setWidth (const double newWidth) noexcept  { w = newWidth; }
+    void setWidth (double newWidth) noexcept        { w = newWidth; }
 
     /** Returns the mode used to calculate the height.
         To change any of the modes, use setModes().
@@ -402,7 +411,7 @@ public:
 
         See getHeight() for the details about what this value means.
     */
-    void setHeight (const double newHeight) noexcept    { h = newHeight; }
+    void setHeight (double newHeight) noexcept      { h = newHeight; }
 
     //==============================================================================
     /** If the size and position are constance, and wouldn't be affected by changes
@@ -420,10 +429,12 @@ public:
     /** Compares two objects. */
     bool operator== (const PositionedRectangle& other) const noexcept
     {
-        return x == other.x && y == other.y
-            && w == other.w && h == other.h
-            && xMode == other.xMode && yMode == other.yMode
-            && wMode == other.wMode && hMode == other.hMode;
+        const auto tie = [] (const PositionedRectangle& r)
+        {
+            return std::tie (r.x, r.y, r.xMode, r.yMode, r.wMode, r.hMode);
+        };
+
+        return tie (*this) == tie (other);
     }
 
     /** Compares two objects. */
