@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -89,7 +98,7 @@ public:
     {
     }
 
-    bool perform()
+    bool perform() override
     {
         showCorrectTab();
         PaintElement* newElement = routine.addElementFromXml (*xml, -1, false);
@@ -100,14 +109,14 @@ public:
         return indexAdded >= 0;
     }
 
-    bool undo()
+    bool undo() override
     {
         showCorrectTab();
         routine.removeElement (routine.getElement (indexAdded), false);
         return true;
     }
 
-    int getSizeInUnits()    { return 10; }
+    int getSizeInUnits() override    { return 10; }
 
     int indexAdded;
 
@@ -172,21 +181,21 @@ public:
         oldIndex = routine.indexOfElement (element);
     }
 
-    bool perform()
+    bool perform() override
     {
         showCorrectTab();
         routine.removeElement (getElement(), false);
         return true;
     }
 
-    bool undo()
+    bool undo() override
     {
         PaintElement* newElement = routine.addElementFromXml (*xml, oldIndex, false);
         showCorrectTab();
         return newElement != nullptr;
     }
 
-    int getSizeInUnits()    { return 10; }
+    int getSizeInUnits() override    { return 10; }
 
 private:
     std::unique_ptr<XmlElement> xml;
@@ -228,7 +237,7 @@ public:
         oldIndex = routine.indexOfElement (element);
     }
 
-    bool perform()
+    bool perform() override
     {
         showCorrectTab();
 
@@ -238,7 +247,7 @@ public:
         return true;
     }
 
-    bool undo()
+    bool undo() override
     {
         showCorrectTab();
         routine.moveElementZOrder (newIndex, oldIndex);
@@ -361,7 +370,7 @@ void PaintRoutine::selectedToFront()
     const SelectedItemSet<PaintElement*> temp (selectedElements);
 
     for (int i = temp.getNumSelected(); --i >= 0;)
-        elementToFront (temp.getSelectedItem(i), true);
+        elementToFront (temp.getSelectedItem (i), true);
 }
 
 void PaintRoutine::selectedToBack()
@@ -369,7 +378,7 @@ void PaintRoutine::selectedToBack()
     const SelectedItemSet<PaintElement*> temp (selectedElements);
 
     for (int i = 0; i < temp.getNumSelected(); ++i)
-        elementToBack (temp.getSelectedItem(i), true);
+        elementToBack (temp.getSelectedItem (i), true);
 }
 
 void PaintRoutine::alignTop()
