@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -266,14 +275,20 @@ void LibraryModule::findBrowseableFiles (const File& folder, Array<File>& filesF
 
 bool LibraryModule::CompileUnit::isNeededForExporter (ProjectExporter& exporter) const
 {
-    if ((hasSuffix (file, "_OSX")        && ! exporter.isOSX())
-     || (hasSuffix (file, "_iOS")        && ! exporter.isiOS())
-     || (hasSuffix (file, "_Windows")    && ! exporter.isWindows())
-     || (hasSuffix (file, "_Linux")      && ! exporter.isLinux())
-     || (hasSuffix (file, "_Android")    && ! exporter.isAndroid()))
-        return false;
+    const auto trimmedFileNameLowercase = file.getFileNameWithoutExtension().toLowerCase();
 
-    auto targetType = Project::getTargetTypeFromFilePath (file, false);
+    const std::tuple<const char*, bool> shouldBuildForSuffix[] { { "_android", exporter.isAndroid() },
+                                                                 { "_ios",     exporter.isiOS() },
+                                                                 { "_linux",   exporter.isLinux() },
+                                                                 { "_mac",     exporter.isOSX() },
+                                                                 { "_osx",     exporter.isOSX() },
+                                                                 { "_windows", exporter.isWindows() } };
+
+    for (const auto& [suffix, shouldBuild] : shouldBuildForSuffix)
+        if (trimmedFileNameLowercase.endsWith (suffix))
+            return shouldBuild;
+
+    const auto targetType = Project::getTargetTypeFromFilePath (file, false);
 
     if (targetType != build_tools::ProjectType::Target::unspecified && ! exporter.shouldBuildTargetType (targetType))
         return false;
@@ -287,14 +302,6 @@ String LibraryModule::CompileUnit::getFilenameForProxyFile() const
     return "include_" + file.getFileName();
 }
 
-bool LibraryModule::CompileUnit::hasSuffix (const File& f, const char* suffix)
-{
-    auto fileWithoutSuffix = f.getFileNameWithoutExtension() + ".";
-
-    return fileWithoutSuffix.containsIgnoreCase (suffix + String ("."))
-             || fileWithoutSuffix.containsIgnoreCase (suffix + String ("_"));
-}
-
 Array<LibraryModule::CompileUnit> LibraryModule::getAllCompileUnits (build_tools::ProjectType::Target::Type forTarget) const
 {
     auto files = getFolder().findChildFiles (File::findFiles, false);
@@ -687,19 +694,24 @@ void EnabledModulesList::addModuleOfferingToCopy (const File& f, bool isFromUser
 
     if (! m.isValid())
     {
-        AlertWindow::showMessageBoxAsync (MessageBoxIconType::InfoIcon,
-                                          "Add Module", "This wasn't a valid module folder!");
+        auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::InfoIcon,
+                                                         "Add Module",
+                                                         "This wasn't a valid module folder!");
+        messageBox = AlertWindow::showScopedAsync (options, nullptr);
         return;
     }
 
     if (isModuleEnabled (m.getID()))
     {
-        AlertWindow::showMessageBoxAsync (MessageBoxIconType::InfoIcon,
-                                          "Add Module", "The project already contains this module!");
+        auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::InfoIcon,
+                                                         "Add Module",
+                                                         "The project already contains this module!");
+        messageBox = AlertWindow::showScopedAsync (options, nullptr);
         return;
     }
 
-    addModule (m.getModuleFolder(), areMostModulesCopiedLocally(),
+    addModule (m.getModuleFolder(),
+               areMostModulesCopiedLocally(),
                isFromUserSpecifiedFolder ? false : areMostModulesUsingGlobalPath());
 }
 
@@ -709,7 +721,7 @@ void EnabledModulesList::removeModule (String moduleID) // must be pass-by-value
         const ScopedLock sl (stateLock);
 
         for (auto i = state.getNumChildren(); --i >= 0;)
-            if (state.getChild(i) [Ids::ID] == moduleID)
+            if (state.getChild (i) [Ids::ID] == moduleID)
                 state.removeChild (i, getUndoManager());
     }
 
