@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -266,7 +275,7 @@ void ProjectContentComponent::setScrollableEditorComponent (std::unique_ptr<Comp
 {
     jassert (component.get() != nullptr);
 
-    class ContentViewport  : public Component
+    class ContentViewport final : public Component
     {
     public:
         ContentViewport (std::unique_ptr<Component> content)
@@ -318,25 +327,26 @@ void ProjectContentComponent::closeDocument()
         hideEditor();
 }
 
-static void showSaveWarning (OpenDocumentManager::Document* currentDocument)
+static ScopedMessageBox showSaveWarning (OpenDocumentManager::Document* currentDocument)
 {
-    AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
-                                      TRANS("Save failed!"),
-                                      TRANS("Couldn't save the file:")
-                                          + "\n" + currentDocument->getFile().getFullPathName());
+    auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::WarningIcon,
+                                                     TRANS ("Save failed!"),
+                                                     TRANS ("Couldn't save the file:")
+                                                         + "\n" + currentDocument->getFile().getFullPathName());
+    return AlertWindow::showScopedAsync (options, nullptr);
 }
 
 void ProjectContentComponent::saveDocumentAsync()
 {
     if (currentDocument != nullptr)
     {
-        currentDocument->saveAsync ([parent = SafePointer<ProjectContentComponent> { this }] (bool savedSuccessfully)
+        currentDocument->saveAsync ([parent = SafePointer { this }] (bool savedSuccessfully)
         {
             if (parent == nullptr)
                 return;
 
             if (! savedSuccessfully)
-                showSaveWarning (parent->currentDocument);
+                parent->messageBox = showSaveWarning (parent->currentDocument);
 
             parent->refreshProjectTreeFileStatuses();
         });
@@ -351,13 +361,13 @@ void ProjectContentComponent::saveAsAsync()
 {
     if (currentDocument != nullptr)
     {
-        currentDocument->saveAsAsync ([parent = SafePointer<ProjectContentComponent> { this }] (bool savedSuccessfully)
+        currentDocument->saveAsAsync ([parent = SafePointer { this }] (bool savedSuccessfully)
         {
             if (parent == nullptr)
                 return;
 
             if (! savedSuccessfully)
-                showSaveWarning (parent->currentDocument);
+                parent->messageBox = showSaveWarning (parent->currentDocument);
 
             parent->refreshProjectTreeFileStatuses();
         });
@@ -600,7 +610,7 @@ void ProjectContentComponent::showTranslationTool()
 }
 
 //==============================================================================
-struct AsyncCommandRetrier  : public Timer
+struct AsyncCommandRetrier final : public Timer
 {
     AsyncCommandRetrier (const ApplicationCommandTarget::InvocationInfo& i)  : info (i)
     {
