@@ -1,31 +1,39 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
 
 #pragma once
 
-#include "../Application/UserAccount/jucer_LicenseController.h"
 #include "Modules/jucer_AvailableModulesList.h"
 
 class ProjectExporter;
@@ -41,7 +49,6 @@ namespace ProjectMessages
 
         DECLARE_ID (projectMessages);
 
-        DECLARE_ID (incompatibleLicense);
         DECLARE_ID (cppStandard);
         DECLARE_ID (moduleNotFound);
         DECLARE_ID (jucePath);
@@ -51,6 +58,7 @@ namespace ProjectMessages
         DECLARE_ID (newVersionAvailable);
         DECLARE_ID (pluginCodeInvalid);
         DECLARE_ID (manufacturerCodeInvalid);
+        DECLARE_ID (deprecatedExporter);
 
         DECLARE_ID (notification);
         DECLARE_ID (warning);
@@ -62,14 +70,17 @@ namespace ProjectMessages
 
     inline Identifier getTypeForMessage (const Identifier& message)
     {
-        static Identifier warnings[] = { Ids::incompatibleLicense, Ids::cppStandard, Ids::moduleNotFound,
-                                         Ids::jucePath, Ids::jucerFileModified, Ids::missingModuleDependencies,
-                                         Ids::oldProjucer, Ids::pluginCodeInvalid, Ids::manufacturerCodeInvalid };
+        static Identifier warnings[] = { Ids::cppStandard, Ids::moduleNotFound, Ids::jucePath,
+                                         Ids::jucerFileModified, Ids::missingModuleDependencies,
+                                         Ids::oldProjucer, Ids::pluginCodeInvalid, Ids::manufacturerCodeInvalid,
+                                         Ids::deprecatedExporter };
 
         if (std::find (std::begin (warnings), std::end (warnings), message) != std::end (warnings))
             return Ids::warning;
 
-        if (message == Ids::newVersionAvailable)
+        static Identifier notifications[] = { Ids::newVersionAvailable };
+
+        if (std::find (std::begin (notifications), std::end (notifications), message) != std::end (notifications))
             return Ids::notification;
 
         jassertfalse;
@@ -78,7 +89,6 @@ namespace ProjectMessages
 
     inline String getTitleForMessage (const Identifier& message)
     {
-        if (message == Ids::incompatibleLicense)        return "Incompatible License and Splash Screen Setting";
         if (message == Ids::cppStandard)                return "C++ Standard";
         if (message == Ids::moduleNotFound)             return "Module Not Found";
         if (message == Ids::jucePath)                   return "JUCE Path";
@@ -88,6 +98,7 @@ namespace ProjectMessages
         if (message == Ids::newVersionAvailable)        return "New Version Available";
         if (message == Ids::pluginCodeInvalid)          return "Invalid Plugin Code";
         if (message == Ids::manufacturerCodeInvalid)    return "Invalid Manufacturer Code";
+        if (message == Ids::deprecatedExporter)         return "Deprecated Exporter";
 
         jassertfalse;
         return {};
@@ -95,7 +106,6 @@ namespace ProjectMessages
 
     inline String getDescriptionForMessage (const Identifier& message)
     {
-        if (message == Ids::incompatibleLicense)        return "Save and export is disabled.";
         if (message == Ids::cppStandard)                return "Module(s) have a higher C++ standard requirement than the project.";
         if (message == Ids::moduleNotFound)             return "Module(s) could not be found at the specified paths.";
         if (message == Ids::jucePath)                   return "The path to your JUCE folder is incorrect.";
@@ -105,6 +115,7 @@ namespace ProjectMessages
         if (message == Ids::newVersionAvailable)        return "A new version of JUCE is available to download.";
         if (message == Ids::pluginCodeInvalid)          return "The plugin code should be exactly four characters in length.";
         if (message == Ids::manufacturerCodeInvalid)    return "The manufacturer code should be exactly four characters in length.";
+        if (message == Ids::deprecatedExporter)         return "The project includes a deprecated exporter.";
 
         jassertfalse;
         return {};
@@ -113,14 +124,72 @@ namespace ProjectMessages
     using MessageAction = std::pair<String, std::function<void()>>;
 }
 
+// Can be shared between multiple classes wanting to create a MessageBox. Ensures that there is one
+// MessageBox active at a given time.
+class MessageBoxQueue : private AsyncUpdater
+{
+public:
+    struct Listener
+    {
+        using CreatorFunction = std::function<ScopedMessageBox (MessageBoxOptions, std::function<void (int)>)>;
+
+        virtual ~Listener() = default;
+
+        virtual void canCreateMessageBox (CreatorFunction) = 0;
+    };
+
+    void handleAsyncUpdate() override
+    {
+        schedule();
+    }
+
+    auto addListener (Listener& l)
+    {
+        triggerAsyncUpdate();
+        return listeners.addScoped (l);
+    }
+
+private:
+    ScopedMessageBox create (MessageBoxOptions options, std::function<void (int)> callback)
+    {
+        hasActiveMessageBox = true;
+
+        return AlertWindow::showScopedAsync (options, [this, cb = std::move (callback)] (int result)
+                                             {
+                                                 cb (result);
+                                                 hasActiveMessageBox = false;
+                                                 triggerAsyncUpdate();
+                                             });
+    }
+
+    void schedule()
+    {
+        if (hasActiveMessageBox)
+            return;
+
+        auto& currentListeners = listeners.getListeners();
+
+        if (! currentListeners.isEmpty())
+        {
+            currentListeners[0]->canCreateMessageBox ([this] (auto o, auto c)
+                                                              {
+                                                                  return create (o, c);
+                                                              });
+        }
+    }
+
+    ListenerList<Listener> listeners;
+    bool hasActiveMessageBox = false;
+};
+
 enum class Async { no, yes };
 
 //==============================================================================
-class Project  : public FileBasedDocument,
-                 private ValueTree::Listener,
-                 private LicenseController::LicenseStateListener,
-                 private ChangeListener,
-                 private AvailableModulesList::Listener
+class Project final : public FileBasedDocument,
+                      private ValueTree::Listener,
+                      private ChangeListener,
+                      private AvailableModulesList::Listener,
+                      private MessageBoxQueue::Listener
 {
 public:
     //==============================================================================
@@ -159,6 +228,7 @@ public:
     static String getJuceSourceHFilename()                      { return "JuceHeader.h"; }
     static String getJuceLV2DefinesFilename()                   { return "JuceLV2Defines.h"; }
     static String getLV2FileWriterName()                        { return "juce_lv2_helper"; }
+    static String getVST3FileWriterName()                       { return "juce_vst3_helper"; }
 
     //==============================================================================
     template <class FileType>
@@ -217,9 +287,6 @@ public:
     bool shouldIncludeBinaryInJuceHeader() const         { return includeBinaryDataInJuceHeaderValue.get(); }
     String getBinaryDataNamespaceString() const          { return binaryDataNamespaceValue.get(); }
 
-    bool shouldDisplaySplashScreen() const               { return displaySplashScreenValue.get(); }
-    String getSplashScreenColourString() const           { return splashScreenColourValue.get(); }
-
     static StringArray getCppStandardStrings()           { return { "C++17", "C++20", "Use Latest" }; }
     static Array<var> getCppStandardVars()               { return { "17",    "20",    "latest" }; }
 
@@ -339,19 +406,17 @@ public:
     String getLV2URI() const        { return pluginLV2URIValue.get(); }
 
     //==============================================================================
-    bool isAUPluginHost();
-    bool isVSTPluginHost();
-    bool isVST3PluginHost();
-    bool isLV2PluginHost();
-    bool isARAPluginHost();
+    bool isAUPluginHost()   const;
+    bool isVSTPluginHost()  const;
+    bool isVST3PluginHost() const;
+    bool isLV2PluginHost()  const;
+    bool isARAPluginHost()  const;
 
     //==============================================================================
     bool shouldBuildTargetType (build_tools::ProjectType::Target::Type targetType) const noexcept;
     static build_tools::ProjectType::Target::Type getTargetTypeFromFilePath (const File& file, bool returnSharedTargetIfNoValidSuffix);
 
     //==============================================================================
-    void updateDeprecatedProjectSettingsInteractively();
-
     StringPairArray getAppConfigDefs();
     StringPairArray getAudioPluginFlags() const;
 
@@ -494,7 +559,10 @@ public:
     bool isConfigFlagEnabled (const String& name, bool defaultIsEnabled = false) const;
 
     //==============================================================================
-    EnabledModulesList& getEnabledModules();
+    void createEnabledModulesList();
+
+          EnabledModulesList& getEnabledModules();
+    const EnabledModulesList& getEnabledModules() const;
 
     AvailableModulesList& getExporterPathsModulesList()  { return exporterPathsModulesList; }
     void rescanExporterPathModules (bool async = false);
@@ -536,10 +604,11 @@ public:
     std::vector<ProjectMessages::MessageAction> getMessageActions (const Identifier& message);
 
     //==============================================================================
-    bool hasIncompatibleLicenseTypeAndSplashScreenSetting() const;
     bool isFileModificationCheckPending() const;
     bool isSaveAndExportDisabled() const;
 
+    MessageBoxQueue messageBoxQueue;
+
 private:
     //==============================================================================
     void valueTreePropertyChanged (ValueTree&, const Identifier&) override;
@@ -547,8 +616,17 @@ private:
     void valueTreeChildRemoved (ValueTree&, ValueTree&, int) override;
     void valueTreeChildOrderChanged (ValueTree&, int, int) override;
 
+    void valueTreeChildAddedOrRemoved (ValueTree&, ValueTree&);
+
     //==============================================================================
-    struct ProjectFileModificationPoller  : private Timer
+    void canCreateMessageBox (CreatorFunction) override;
+
+    //==============================================================================
+    template <typename This>
+    static auto& getEnabledModulesImpl (This&);
+
+    //==============================================================================
+    struct ProjectFileModificationPoller final : private Timer
     {
         ProjectFileModificationPoller (Project& p);
         bool isCheckPending() const noexcept  { return pending; }
@@ -568,9 +646,9 @@ private:
     ValueTree projectRoot  { Ids::JUCERPROJECT };
 
     ValueTreePropertyWithDefault projectNameValue, projectUIDValue, projectLineFeedValue, projectTypeValue, versionValue, bundleIdentifierValue, companyNameValue,
-                                 companyCopyrightValue, companyWebsiteValue, companyEmailValue, displaySplashScreenValue, splashScreenColourValue, cppStandardValue,
-                                 headerSearchPathsValue, preprocessorDefsValue, userNotesValue, maxBinaryFileSizeValue, includeBinaryDataInJuceHeaderValue, binaryDataNamespaceValue,
-                                 compilerFlagSchemesValue, postExportShellCommandPosixValue, postExportShellCommandWinValue, useAppConfigValue, addUsingNamespaceToJuceHeader;
+                                 companyCopyrightValue, companyWebsiteValue, companyEmailValue, cppStandardValue, headerSearchPathsValue, preprocessorDefsValue,
+                                 userNotesValue, maxBinaryFileSizeValue, includeBinaryDataInJuceHeaderValue, binaryDataNamespaceValue, compilerFlagSchemesValue,
+                                 postExportShellCommandPosixValue, postExportShellCommandWinValue, useAppConfigValue, addUsingNamespaceToJuceHeader;
 
     ValueTreePropertyWithDefault pluginFormatsValue, pluginNameValue, pluginDescriptionValue, pluginManufacturerValue, pluginManufacturerCodeValue,
                                  pluginCodeValue, pluginChannelConfigsValue, pluginCharacteristicsValue, pluginAUExportPrefixValue, pluginAAXIdentifierValue,
@@ -630,14 +708,13 @@ private:
     void updateOldModulePaths();
 
     //==============================================================================
-    void licenseStateChanged() override;
     void changeListenerCallback (ChangeBroadcaster*) override;
     void availableModulesChanged (AvailableModulesList*) override;
 
-    void updateLicenseWarning();
     void updateJUCEPathWarning();
 
     void updateModuleWarnings();
+    void updateExporterWarnings();
     void updateCppStandardWarning (bool showWarning);
     void updateMissingModuleDependenciesWarning (bool showWarning);
     void updateOldProjucerWarning (bool showWarning);
@@ -653,6 +730,10 @@ private:
     std::unique_ptr<FileChooser> chooser;
     std::unique_ptr<ProjectSaver> saver;
 
+    std::optional<MessageBoxOptions> exporterRemovalMessageBoxOptions;
+    ErasedScopeGuard messageBoxQueueListenerScope;
+    ScopedMessageBox messageBox;
+
     //==============================================================================
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Project)
     JUCE_DECLARE_WEAK_REFERENCEABLE (Project)
