@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -234,6 +243,32 @@ build_tools::RelativePath ProjectExporter::rebaseFromProjectFolderToBuildTarget
     return path.rebased (project.getProjectFolder(), getTargetFolder(), build_tools::RelativePath::buildTargetFolder);
 }
 
+build_tools::RelativePath ProjectExporter::rebaseFromBuildTargetToProjectFolder (const build_tools::RelativePath& path) const
+{
+    jassert (path.getRoot() == build_tools::RelativePath::buildTargetFolder);
+    return path.rebased (getTargetFolder(), project.getProjectFolder(), build_tools::RelativePath::projectFolder);
+}
+
+File ProjectExporter::resolveRelativePath (const build_tools::RelativePath& path) const
+{
+    if (path.isAbsolute())
+        return path.toUnixStyle();
+
+    switch (path.getRoot())
+    {
+        case build_tools::RelativePath::buildTargetFolder:
+            return getTargetFolder().getChildFile (path.toUnixStyle());
+
+        case build_tools::RelativePath::projectFolder:
+            return project.getProjectFolder().getChildFile (path.toUnixStyle());
+
+        case build_tools::RelativePath::unknown:
+            jassertfalse;
+    }
+
+    return path.toUnixStyle();
+}
+
 bool ProjectExporter::shouldFileBeCompiledByDefault (const File& file) const
 {
     return file.hasFileExtension (cOrCppFileExtensions)
@@ -425,16 +460,11 @@ build_tools::RelativePath ProjectExporter::getInternalVST3SDKPath()
 
 void ProjectExporter::addAAXFoldersToPath()
 {
-    auto aaxFolder = getAAXPathString();
+    const auto aaxFolder = getAAXPathRelative();
 
-    if (aaxFolder.isNotEmpty())
-    {
-        build_tools::RelativePath aaxFolderPath (aaxFolder, build_tools::RelativePath::projectFolder);
-
-        addToExtraSearchPaths (aaxFolderPath);
-        addToExtraSearchPaths (aaxFolderPath.getChildFile ("Interfaces"));
-        addToExtraSearchPaths (aaxFolderPath.getChildFile ("Interfaces").getChildFile ("ACF"));
-    }
+    addToExtraSearchPaths (aaxFolder);
+    addToExtraSearchPaths (aaxFolder.getChildFile ("Interfaces"));
+    addToExtraSearchPaths (aaxFolder.getChildFile ("Interfaces").getChildFile ("ACF"));
 }
 
 void ProjectExporter::addARAFoldersToPath()
@@ -497,6 +527,8 @@ void ProjectExporter::addTargetSpecificPreprocessorDefs (StringPairArray& defs,
     {
         defs.set ("JucePlugin_Enable_ARA", "1");
     }
+
+    linuxSubprocessHelperProperties.setCompileDefinitionIfNecessary (defs);
 }
 
 void ProjectExporter::addDefaultPreprocessorDefs (StringPairArray& defs) const
@@ -545,7 +577,7 @@ Project::Item& ProjectExporter::getModulesGroup()
 }
 
 //==============================================================================
-static bool isWebBrowserComponentEnabled (Project& project)
+static bool isWebBrowserComponentEnabled (const Project& project)
 {
     static String guiExtrasModule ("juce_gui_extra");
 
@@ -608,6 +640,7 @@ void ProjectExporter::addToModuleLibPaths (const build_tools::RelativePath& path
 
 void ProjectExporter::addToExtraSearchPaths (const build_tools::RelativePath& pathFromProjectFolder, int index)
 {
+    jassert (pathFromProjectFolder.getRoot() == build_tools::RelativePath::projectFolder);
     addProjectPathToBuildPathList (extraSearchPaths, pathFromProjectFolder, index);
 }
 
@@ -893,17 +926,35 @@ ProjectExporter::BuildConfiguration::BuildConfiguration (Project& p, const Value
      configLinkerFlagsValue        (config, Ids::extraLinkerFlags,         getUndoManager())
 {
     auto& llvmFlags = recommendedCompilerWarningFlags[CompilerNames::llvm] = BuildConfiguration::CompilerWarningFlags::getRecommendedForGCCAndLLVM();
-    llvmFlags.common.addArray ({ "-Wshorten-64-to-32", "-Wconversion", "-Wint-conversion",
-                                 "-Wconditional-uninitialized", "-Wconstant-conversion", "-Wbool-conversion",
-                                 "-Wextra-semi", "-Wshift-sign-overflow",
-                                 "-Wshadow-all", "-Wnullable-to-nonnull-conversion",
-                                 "-Wmissing-prototypes" });
-    llvmFlags.cpp.addArray ({ "-Wunused-private-field", "-Winconsistent-missing-destructor-override" });
-    llvmFlags.objc.addArray ({ "-Wunguarded-availability", "-Wunguarded-availability-new" });
+
+    llvmFlags.common.addArray ({ "-Wshadow-all",
+                                 "-Wshorten-64-to-32",
+                                 "-Wconversion",
+                                 "-Wint-conversion",
+                                 "-Wconditional-uninitialized",
+                                 "-Wconstant-conversion",
+                                 "-Wbool-conversion",
+                                 "-Wextra-semi",
+                                 "-Wshift-sign-overflow",
+                                 "-Wmissing-prototypes",
+                                 "-Wnullable-to-nonnull-conversion",
+                                 "-Wpedantic",
+                                 "-Wdeprecated" });
+
+    llvmFlags.cpp.addArray ({ "-Wunused-private-field",
+                              "-Winconsistent-missing-destructor-override" });
+
+    llvmFlags.objc.addArray ({ "-Wunguarded-availability",
+                               "-Wunguarded-availability-new" });
 
     auto& gccFlags = recommendedCompilerWarningFlags[CompilerNames::gcc] = BuildConfiguration::CompilerWarningFlags::getRecommendedForGCCAndLLVM();
-    gccFlags.common.addArray ({ "-Wextra", "-Wsign-compare", "-Wno-implicit-fallthrough", "-Wno-maybe-uninitialized",
-                                "-Wredundant-decls", "-Wno-strict-overflow", "-Wshadow" });
+    gccFlags.common.addArray ({ "-Wextra",
+                                "-Wsign-compare",
+                                "-Wno-implicit-fallthrough",
+                                "-Wno-maybe-uninitialized",
+                                "-Wredundant-decls",
+                                "-Wno-strict-overflow",
+                                "-Wshadow" });
 }
 
 String ProjectExporter::BuildConfiguration::getGCCOptimisationFlag() const
@@ -1083,3 +1134,115 @@ String ProjectExporter::getExternalLibraryFlags (const BuildConfiguration& confi
 
     return {};
 }
+
+//==============================================================================
+LinuxSubprocessHelperProperties::LinuxSubprocessHelperProperties (ProjectExporter& projectExporter)
+    : owner (projectExporter)
+{}
+
+bool LinuxSubprocessHelperProperties::shouldUseLinuxSubprocessHelper() const
+{
+    const auto& project = owner.getProject();
+    const auto& projectType = project.getProjectType();
+
+    return    owner.isLinux()
+           && isWebBrowserComponentEnabled (project)
+           && ! (projectType.isCommandLineApp())
+           && ! (projectType.isGUIApplication());
+}
+
+void LinuxSubprocessHelperProperties::deployLinuxSubprocessHelperSourceFilesIfNecessary() const
+{
+    if (shouldUseLinuxSubprocessHelper())
+    {
+        const auto deployHelperSourceFile = [] (auto& sourcePath, auto& contents)
+        {
+            if (! sourcePath.isRoot() && ! sourcePath.getParentDirectory().exists())
+            {
+                sourcePath.getParentDirectory().createDirectory();
+            }
+
+            build_tools::overwriteFileIfDifferentOrThrow (sourcePath, contents);
+        };
+
+        const std::pair<File, const char*> sources[]
+        {
+            { owner.resolveRelativePath (getSimpleBinaryBuilderSource()),   BinaryData::juce_SimpleBinaryBuilder_cpp   },
+            { owner.resolveRelativePath (getLinuxSubprocessHelperSource()), BinaryData::juce_LinuxSubprocessHelper_cpp }
+        };
+
+        for (const auto& [path, source] : sources)
+        {
+            deployHelperSourceFile (path, source);
+        }
+    }
+}
+
+build_tools::RelativePath LinuxSubprocessHelperProperties::getLinuxSubprocessHelperSource() const
+{
+    return build_tools::RelativePath { "make_helpers", build_tools::RelativePath::buildTargetFolder }
+        .getChildFile ("juce_LinuxSubprocessHelper.cpp");
+}
+
+void LinuxSubprocessHelperProperties::setCompileDefinitionIfNecessary (StringPairArray& defs) const
+{
+    if (shouldUseLinuxSubprocessHelper())
+        defs.set (useLinuxSubprocessHelperCompileDefinition, "1");
+}
+
+build_tools::RelativePath LinuxSubprocessHelperProperties::getSimpleBinaryBuilderSource() const
+{
+    return build_tools::RelativePath { "make_helpers", build_tools::RelativePath::buildTargetFolder }
+                            .getChildFile ("juce_SimpleBinaryBuilder.cpp");
+}
+
+build_tools::RelativePath LinuxSubprocessHelperProperties::getLinuxSubprocessHelperBinaryDataSource() const
+{
+    return build_tools::RelativePath ("pre_build", juce::build_tools::RelativePath::buildTargetFolder)
+                            .getChildFile ("juce_LinuxSubprocessHelperBinaryData.cpp");
+}
+
+void LinuxSubprocessHelperProperties::addToExtraSearchPathsIfNecessary() const
+{
+    if (shouldUseLinuxSubprocessHelper())
+    {
+        const auto subprocessHelperBinaryDir = getLinuxSubprocessHelperBinaryDataSource().getParentDirectory();
+        owner.addToExtraSearchPaths (owner.rebaseFromBuildTargetToProjectFolder (subprocessHelperBinaryDir));
+    }
+}
+
+std::optional<String> LinuxSubprocessHelperProperties::getParentDirectoryRelativeToBuildTargetFolder (build_tools::RelativePath rp)
+{
+    jassert (rp.getRoot() == juce::build_tools::RelativePath::buildTargetFolder);
+    const auto parentDir = rp.getParentDirectory().toUnixStyle();
+    return parentDir == rp.toUnixStyle() ? std::nullopt : std::make_optional (parentDir);
+}
+
+String LinuxSubprocessHelperProperties::makeSnakeCase (const String& s)
+{
+    String result;
+    result.preallocateBytes (128);
+
+    bool previousCharacterUnderscore = false;
+
+    for (const auto c : s)
+    {
+        if (   CharacterFunctions::isUpperCase (c)
+            && result.length() != 0
+            && ! (previousCharacterUnderscore))
+        {
+            result << "_";
+        }
+
+        result << CharacterFunctions::toLowerCase (c);
+
+        previousCharacterUnderscore = c == '_';
+    }
+
+    return result;
+}
+
+String LinuxSubprocessHelperProperties::getBinaryNameFromSource (const build_tools::RelativePath& rp)
+{
+    return makeSnakeCase (rp.getFileNameWithoutExtension());
+}
