@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -32,8 +41,39 @@
 
 class ProjectSaver;
 
+class LinuxSubprocessHelperProperties
+{
+public:
+    explicit LinuxSubprocessHelperProperties (ProjectExporter& projectExporter);
+
+    bool shouldUseLinuxSubprocessHelper() const;
+
+    void deployLinuxSubprocessHelperSourceFilesIfNecessary() const;
+
+    build_tools::RelativePath getLinuxSubprocessHelperSource() const;
+
+    void setCompileDefinitionIfNecessary (StringPairArray& defs) const;
+
+    build_tools::RelativePath getSimpleBinaryBuilderSource() const;
+
+    build_tools::RelativePath getLinuxSubprocessHelperBinaryDataSource() const;
+
+    void addToExtraSearchPathsIfNecessary() const;
+
+    static std::optional<String> getParentDirectoryRelativeToBuildTargetFolder (build_tools::RelativePath rp);
+
+    static String makeSnakeCase (const String& s);
+
+    static String getBinaryNameFromSource (const build_tools::RelativePath& rp);
+
+    static constexpr const char* useLinuxSubprocessHelperCompileDefinition = "JUCE_USE_EXTERNAL_TEMPORARY_SUBPROCESS";
+
+private:
+    ProjectExporter& owner;
+};
+
 //==============================================================================
-class ProjectExporter  : private Value::Listener
+class ProjectExporter : private Value::Listener
 {
 public:
     ProjectExporter (Project&, const ValueTree& settings);
@@ -70,7 +110,6 @@ public:
     virtual bool canCopeWithDuplicateFiles() = 0;
     virtual bool supportsUserDefinedConfigurations() const = 0; // false if exporter only supports two configs Debug and Release
     virtual void updateDeprecatedSettings()               {}
-    virtual void updateDeprecatedSettingsInteractively()  {}
     virtual void initialiseDependencyPathValues()         {}
 
     // IDE targeted by exporter
@@ -145,7 +184,16 @@ public:
     bool shouldUseGNUExtensions() const                   { return gnuExtensionsValue.get(); }
 
     String getVSTLegacyPathString() const                 { return vstLegacyPathValueWrapper.getCurrentValue(); }
-    String getAAXPathString() const                       { return aaxPathValueWrapper.getCurrentValue(); }
+
+    auto getAAXPathRelative() const
+    {
+        const String userAaxFolder = aaxPathValueWrapper.getCurrentValue();
+        return userAaxFolder.isNotEmpty()
+             ? build_tools::RelativePath (userAaxFolder, build_tools::RelativePath::projectFolder)
+             : getModuleFolderRelativeToProject ("juce_audio_plugin_client").getChildFile ("AAX")
+                                                                            .getChildFile ("SDK");
+    }
+
     String getARAPathString() const                       { return araPathValueWrapper.getCurrentValue(); }
 
     // NB: this is the path to the parent "modules" folder that contains the named module, not the
@@ -164,6 +212,8 @@ public:
     void updateOldModulePaths();
 
     build_tools::RelativePath rebaseFromProjectFolderToBuildTarget (const build_tools::RelativePath& path) const;
+    build_tools::RelativePath rebaseFromBuildTargetToProjectFolder (const build_tools::RelativePath& path) const;
+    File resolveRelativePath (const build_tools::RelativePath&) const;
     void addToExtraSearchPaths (const build_tools::RelativePath& pathFromProjectFolder, int index = -1);
     void addToModuleLibPaths   (const build_tools::RelativePath& pathFromProjectFolder);
 
@@ -183,11 +233,19 @@ public:
     void createPropertyEditors (PropertyListBuilder&);
     void addSettingsForProjectType (const build_tools::ProjectType&);
 
-    build_tools::RelativePath getLV2TurtleDumpProgramSource() const
+    build_tools::RelativePath getLV2HelperProgramSource() const
     {
         return getModuleFolderRelativeToProject ("juce_audio_plugin_client")
                .getChildFile ("LV2")
-               .getChildFile ("juce_LV2TurtleDumpProgram.cpp");
+               .getChildFile ("juce_LV2ManifestHelper.cpp");
+    }
+
+    build_tools::RelativePath getVST3HelperProgramSource() const
+    {
+        const auto suffix = isOSX() ? "mm" : "cpp";
+        return getModuleFolderRelativeToProject ("juce_audio_plugin_client")
+               .getChildFile ("VST3")
+               .getChildFile (String ("juce_VST3ManifestHelper.") + suffix);
     }
 
     //==============================================================================
@@ -220,7 +278,10 @@ public:
     StringArray moduleLibSearchPaths;
 
     //==============================================================================
-    class BuildConfiguration  : public ReferenceCountedObject
+    const LinuxSubprocessHelperProperties linuxSubprocessHelperProperties { *this };
+
+    //==============================================================================
+    class BuildConfiguration : public ReferenceCountedObject
     {
     public:
         BuildConfiguration (Project& project, const ValueTree& configNode, const ProjectExporter&);
@@ -284,10 +345,26 @@ public:
             static CompilerWarningFlags getRecommendedForGCCAndLLVM()
             {
                 CompilerWarningFlags result;
-                result.common = { "-Wall", "-Wstrict-aliasing", "-Wuninitialized", "-Wunused-parameter",
-                                  "-Wswitch-enum", "-Wsign-conversion", "-Wsign-compare",
-                                  "-Wunreachable-code", "-Wcast-align", "-Wno-ignored-qualifiers" };
-                result.cpp = { "-Woverloaded-virtual", "-Wreorder", "-Wzero-as-null-pointer-constant" };
+                result.common = {
+                    "-Wall",
+                    "-Wcast-align",
+                    "-Wfloat-equal",
+                    "-Wno-ignored-qualifiers",
+                    "-Wsign-compare",
+                    "-Wsign-conversion",
+                    "-Wstrict-aliasing",
+                    "-Wswitch-enum",
+                    "-Wuninitialized",
+                    "-Wunreachable-code",
+                    "-Wunused-parameter",
+                    "-Wmissing-field-initializers"
+                };
+
+                result.cpp = {
+                    "-Woverloaded-virtual",
+                    "-Wreorder",
+                    "-Wzero-as-null-pointer-constant"
+                };
 
                 return result;
             }
