@@ -1,21 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   The code included in this file is provided under the terms of the ISC license
-   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
-   To use, copy, modify, and/or distribute this software for any purpose with or
-   without fee is hereby granted provided that the above copyright notice and
-   this permission notice appear in all copies.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
+
+   Or:
+
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -86,6 +98,252 @@ using uint32    = unsigned int;
   using ssize_t = pointer_sized_int;
 #endif
 
+//==============================================================================
+/** Handy function for avoiding unused variables warning. */
+template <typename... Types>
+void ignoreUnused (Types&&...) noexcept {}
+
+/** Handy function for getting the number of elements in a simple const C array.
+    E.g.
+    @code
+    static int myArray[] = { 1, 2, 3 };
+
+    int numElements = numElementsInArray (myArray) // returns 3
+    @endcode
+*/
+template <typename Type, size_t N>
+constexpr int numElementsInArray (Type (&)[N]) noexcept     { return N; }
+
+//==============================================================================
+// Some useful maths functions that aren't always present with all compilers and build settings.
+
+/** Using juce_hypot is easier than dealing with the different types of hypot function
+    that are provided by the various platforms and compilers. */
+template <typename Type>
+Type juce_hypot (Type a, Type b) noexcept
+{
+   #if JUCE_MSVC
+    return static_cast<Type> (_hypot (a, b));
+   #else
+    return static_cast<Type> (hypot (a, b));
+   #endif
+}
+
+#ifndef DOXYGEN
+template <>
+inline float juce_hypot (float a, float b) noexcept
+{
+   #if JUCE_MSVC
+    return _hypotf (a, b);
+   #else
+    return hypotf (a, b);
+   #endif
+}
+#endif
+
+//==============================================================================
+/** Commonly used mathematical constants
+
+    @tags{Core}
+*/
+template <typename FloatType>
+struct MathConstants
+{
+    /** A predefined value for Pi */
+    static constexpr FloatType pi = static_cast<FloatType> (3.141592653589793238L);
+
+    /** A predefined value for 2 * Pi */
+    static constexpr FloatType twoPi = static_cast<FloatType> (2 * 3.141592653589793238L);
+
+    /** A predefined value for Pi / 2 */
+    static constexpr FloatType halfPi = static_cast<FloatType> (3.141592653589793238L / 2);
+
+    /** A predefined value for Euler's number */
+    static constexpr FloatType euler = static_cast<FloatType> (2.71828182845904523536L);
+
+    /** A predefined value for sqrt (2) */
+    static constexpr FloatType sqrt2 = static_cast<FloatType> (1.4142135623730950488L);
+};
+
+#ifndef DOXYGEN
+/** A double-precision constant for pi. */
+[[deprecated ("This is deprecated in favour of MathConstants<double>::pi.")]]
+const constexpr double  double_Pi  = MathConstants<double>::pi;
+
+/** A single-precision constant for pi. */
+[[deprecated ("This is deprecated in favour of MathConstants<float>::pi.")]]
+const constexpr float   float_Pi   = MathConstants<float>::pi;
+#endif
+
+/** Converts an angle in degrees to radians. */
+template <typename FloatType>
+constexpr FloatType degreesToRadians (FloatType degrees) noexcept     { return degrees * (MathConstants<FloatType>::pi / FloatType (180)); }
+
+/** Converts an angle in radians to degrees. */
+template <typename FloatType>
+constexpr FloatType radiansToDegrees (FloatType radians) noexcept     { return radians * (FloatType (180) / MathConstants<FloatType>::pi); }
+
+//==============================================================================
+/** The isfinite() method seems to vary between platforms, so this is a
+    platform-independent function for it.
+*/
+template <typename NumericType>
+bool juce_isfinite (NumericType value) noexcept
+{
+    if constexpr (std::numeric_limits<NumericType>::has_infinity
+                  || std::numeric_limits<NumericType>::has_quiet_NaN
+                  || std::numeric_limits<NumericType>::has_signaling_NaN)
+    {
+        return std::isfinite (value);
+    }
+    else
+    {
+        ignoreUnused (value);
+        return true;
+    }
+}
+
+//==============================================================================
+/** Equivalent to operator==, but suppresses float-equality warnings.
+
+    This allows code to be explicit about float-equality checks that are known to have the correct
+    semantics.
+*/
+template <typename Type>
+constexpr bool exactlyEqual (Type a, Type b)
+{
+    JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wfloat-equal")
+    return a == b;
+    JUCE_END_IGNORE_WARNINGS_GCC_LIKE
+}
+
+/** A class encapsulating both relative and absolute tolerances for use in floating-point comparisons.
+
+    @see approximatelyEqual, absoluteTolerance, relativeTolerance
+
+    @tags{Core}
+*/
+template <typename Type>
+class Tolerance
+{
+public:
+    Tolerance() = default;
+
+    /** Returns a copy of this Tolerance object with a new absolute tolerance.
+
+        If you just need a Tolerance object with an absolute tolerance, it might be worth using the
+        absoluteTolerance() function.
+
+        @see getAbsolute, absoluteTolerance
+    */
+    [[nodiscard]] Tolerance withAbsolute (Type newAbsolute)
+    {
+        return withMember (*this, &Tolerance::absolute, std::abs (newAbsolute));
+    }
+
+    /** Returns a copy of this Tolerance object with a new relative tolerance.
+
+        If you just need a Tolerance object with a relative tolerance, it might be worth using the
+        relativeTolerance() function.
+
+        @see getRelative, relativeTolerance
+    */
+    [[nodiscard]] Tolerance withRelative (Type newRelative)
+    {
+        return withMember (*this, &Tolerance::relative, std::abs (newRelative));
+    }
+
+    [[nodiscard]] Type getAbsolute() const { return absolute; }
+    [[nodiscard]] Type getRelative() const { return relative; }
+
+private:
+    Type absolute{};
+    Type relative{};
+};
+
+/** Returns a type deduced Tolerance object containing only an absolute tolerance.
+
+    @see Tolerance::withAbsolute, approximatelyEqual
+ */
+template <typename Type>
+static Tolerance<Type> absoluteTolerance (Type tolerance)
+{
+    return Tolerance<Type>{}.withAbsolute (tolerance);
+}
+
+/** Returns a type deduced Tolerance object containing only a relative tolerance.
+
+    @see Tolerance::withRelative, approximatelyEqual
+ */
+template <typename Type>
+static Tolerance<Type> relativeTolerance (Type tolerance)
+{
+    return Tolerance<Type>{}.withRelative (tolerance);
+}
+
+
+/** Returns true if the two floating-point numbers are approximately equal.
+
+    If either a or b are not finite, returns exactlyEqual (a, b).
+
+    The default absolute tolerance is equal to the minimum normal value. This ensures
+    differences that are subnormal are always considered equal. It is highly recommend this
+    value is reviewed depending on the calculation being carried out. In general specifying an
+    absolute value is useful when considering values close to zero. For example you might
+    expect sin (pi) to return 0, but what it actually returns is close to the error of the value pi.
+    Therefore, in this example it might be better to set the absolute tolerance to sin (pi).
+
+    The default relative tolerance is equal to the machine epsilon which is the difference between
+    1.0 and the next floating-point value that can be represented by Type. In most cases this value
+    is probably reasonable. This value is multiplied by the largest absolute value of a and b so as
+    to scale relatively according to the input parameters. For example, specifying a relative value
+    of 0.05 will ensure values return equal if the difference between them is less than or equal to
+    5% of the larger of the two absolute values.
+
+    @param a            The first number to compare.
+    @param b            The second number to compare.
+    @param tolerance    An object that represents both absolute and relative tolerances
+                        when evaluating if a and b are equal.
+
+    @see exactlyEqual
+*/
+template <typename Type, std::enable_if_t<std::is_floating_point_v<Type>, int> = 0>
+constexpr bool approximatelyEqual (Type a, Type b,
+                                   Tolerance<Type> tolerance = Tolerance<Type>{}
+                                        .withAbsolute (std::numeric_limits<Type>::min())
+                                        .withRelative (std::numeric_limits<Type>::epsilon()))
+{
+    if (! (juce_isfinite (a) && juce_isfinite (b)))
+        return exactlyEqual (a, b);
+
+    const auto diff = std::abs (a - b);
+
+    return diff <= tolerance.getAbsolute()
+        || diff <= tolerance.getRelative() * std::max (std::abs (a), std::abs (b));
+}
+
+/** Special case for non-floating-point types that returns true if both are exactly equal. */
+template <typename Type, std::enable_if_t<! std::is_floating_point_v<Type>, int> = 0>
+constexpr bool approximatelyEqual (Type a, Type b)
+{
+    return a == b;
+}
+
+//==============================================================================
+/** Returns the next representable value by FloatType in the direction of the largest representable value. */
+template <typename FloatType>
+FloatType nextFloatUp (FloatType value) noexcept
+{
+    return std::nextafter (value, std::numeric_limits<FloatType>::max());
+}
+
+/** Returns the next representable value by FloatType in the direction of the lowest representable value. */
+template <typename FloatType>
+FloatType nextFloatDown (FloatType value) noexcept
+{
+    return std::nextafter (value, std::numeric_limits<FloatType>::lowest());
+}
+
 //==============================================================================
 // Some indispensable min/max functions
 
@@ -126,7 +384,7 @@ constexpr Type jmap (Type value0To1, Type targetRangeMin, Type targetRangeMax)
 template <typename Type>
 Type jmap (Type sourceValue, Type sourceRangeMin, Type sourceRangeMax, Type targetRangeMin, Type targetRangeMax)
 {
-    jassert (sourceRangeMax != sourceRangeMin); // mapping from a range of zero will produce NaN!
+    jassert (! approximatelyEqual (sourceRangeMax, sourceRangeMin)); // mapping from a range of zero will produce NaN!
     return targetRangeMin + ((targetRangeMax - targetRangeMin) * (sourceValue - sourceRangeMin)) / (sourceRangeMax - sourceRangeMin);
 }
 
@@ -317,132 +575,6 @@ bool isWithin (Type a, Type b, Type tolerance) noexcept
     return std::abs (a - b) <= tolerance;
 }
 
-/** Returns true if the two numbers are approximately equal. This is useful for floating-point
-    and double comparisons.
-*/
-template <typename Type>
-bool approximatelyEqual (Type a, Type b) noexcept
-{
-    return std::abs (a - b) <= (std::numeric_limits<Type>::epsilon() * std::max (a, b))
-            || std::abs (a - b) < std::numeric_limits<Type>::min();
-}
-
-//==============================================================================
-/** Handy function for avoiding unused variables warning. */
-template <typename... Types>
-void ignoreUnused (Types&&...) noexcept {}
-
-/** Handy function for getting the number of elements in a simple const C array.
-    E.g.
-    @code
-    static int myArray[] = { 1, 2, 3 };
-
-    int numElements = numElementsInArray (myArray) // returns 3
-    @endcode
-*/
-template <typename Type, size_t N>
-constexpr int numElementsInArray (Type (&)[N]) noexcept     { return N; }
-
-//==============================================================================
-// Some useful maths functions that aren't always present with all compilers and build settings.
-
-/** Using juce_hypot is easier than dealing with the different types of hypot function
-    that are provided by the various platforms and compilers. */
-template <typename Type>
-Type juce_hypot (Type a, Type b) noexcept
-{
-   #if JUCE_MSVC
-    return static_cast<Type> (_hypot (a, b));
-   #else
-    return static_cast<Type> (hypot (a, b));
-   #endif
-}
-
-#ifndef DOXYGEN
-template <>
-inline float juce_hypot (float a, float b) noexcept
-{
-   #if JUCE_MSVC
-    return _hypotf (a, b);
-   #else
-    return hypotf (a, b);
-   #endif
-}
-#endif
-
-//==============================================================================
-/** Commonly used mathematical constants
-
-    @tags{Core}
-*/
-template <typename FloatType>
-struct MathConstants
-{
-    /** A predefined value for Pi */
-    static constexpr FloatType pi = static_cast<FloatType> (3.141592653589793238L);
-
-    /** A predefined value for 2 * Pi */
-    static constexpr FloatType twoPi = static_cast<FloatType> (2 * 3.141592653589793238L);
-
-    /** A predefined value for Pi / 2 */
-    static constexpr FloatType halfPi = static_cast<FloatType> (3.141592653589793238L / 2);
-
-    /** A predefined value for Euler's number */
-    static constexpr FloatType euler = static_cast<FloatType> (2.71828182845904523536L);
-
-    /** A predefined value for sqrt(2) */
-    static constexpr FloatType sqrt2 = static_cast<FloatType> (1.4142135623730950488L);
-};
-
-#ifndef DOXYGEN
-/** A double-precision constant for pi. */
-[[deprecated ("This is deprecated in favour of MathConstants<double>::pi.")]]
-const constexpr double  double_Pi  = MathConstants<double>::pi;
-
-/** A single-precision constant for pi. */
-[[deprecated ("This is deprecated in favour of MathConstants<float>::pi.")]]
-const constexpr float   float_Pi   = MathConstants<float>::pi;
-#endif
-
-/** Converts an angle in degrees to radians. */
-template <typename FloatType>
-constexpr FloatType degreesToRadians (FloatType degrees) noexcept     { return degrees * (MathConstants<FloatType>::pi / FloatType (180)); }
-
-/** Converts an angle in radians to degrees. */
-template <typename FloatType>
-constexpr FloatType radiansToDegrees (FloatType radians) noexcept     { return radians * (FloatType (180) / MathConstants<FloatType>::pi); }
-
-
-//==============================================================================
-/** The isfinite() method seems to vary between platforms, so this is a
-    platform-independent function for it.
-*/
-template <typename NumericType>
-bool juce_isfinite (NumericType) noexcept
-{
-    return true; // Integer types are always finite
-}
-
-template <>
-inline bool juce_isfinite (float value) noexcept
-{
-   #if JUCE_WINDOWS && ! JUCE_MINGW
-    return _finite (value) != 0;
-   #else
-    return std::isfinite (value);
-   #endif
-}
-
-template <>
-inline bool juce_isfinite (double value) noexcept
-{
-   #if JUCE_WINDOWS && ! JUCE_MINGW
-    return _finite (value) != 0;
-   #else
-    return std::isfinite (value);
-   #endif
-}
-
 //==============================================================================
 #if JUCE_MSVC
  #pragma optimize ("t", off)
@@ -679,4 +811,13 @@ namespace TypeHelpers
  [[deprecated ("Use std::abs() instead.")]] inline int64 abs64 (int64 n) noexcept                { return std::abs (n); }
 #endif
 
+/** Converts an enum to its underlying integral type.
+    Similar to std::to_underlying, which is only available in C++23 and above.
+*/
+template <typename T>
+constexpr auto toUnderlyingType (T t) -> std::enable_if_t<std::is_enum_v<T>, std::underlying_type_t<T>>
+{
+    return static_cast<std::underlying_type_t<T>> (t);
+}
+
 } // namespace juce
