@@ -1,31 +1,38 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
 
-namespace juce
-{
-namespace dsp
+namespace juce::dsp
 {
 
 template <typename Element>
@@ -72,7 +79,7 @@ private:
     std::vector<Element> storage;
 };
 
-class BackgroundMessageQueue  : private Thread
+class BackgroundMessageQueue : private Thread
 {
 public:
     explicit BackgroundMessageQueue (int entries)
@@ -647,7 +654,7 @@ static AudioBuffer<float> resampleImpulseResponse (const AudioBuffer<float>& buf
                                                    const double srcSampleRate,
                                                    const double destSampleRate)
 {
-    if (srcSampleRate == destSampleRate)
+    if (approximatelyEqual (srcSampleRate, destSampleRate))
         return buf;
 
     const auto factorReading = srcSampleRate / destSampleRate;
@@ -845,7 +852,7 @@ static void setImpulseResponse (ConvolutionEngineFactory& factory,
 // this object when adding commands to the background message queue.
 // That way, we can avoid dangling references in the background thread in the case
 // that a Convolution instance is deleted before the background message queue.
-class ConvolutionEngineQueue  : public std::enable_shared_from_this<ConvolutionEngineQueue>
+class ConvolutionEngineQueue final : public std::enable_shared_from_this<ConvolutionEngineQueue>
 {
 public:
     ConvolutionEngineQueue (BackgroundMessageQueue& queue,
@@ -1293,5 +1300,4 @@ int Convolution::getCurrentIRSize() const { return pimpl->getCurrentIRSize(); }
 
 int Convolution::getLatency() const { return pimpl->getLatency(); }
 
-} // namespace dsp
-} // namespace juce
+} // namespace juce::dsp
