@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -26,7 +35,7 @@
 namespace juce
 {
 
-struct ColourComponentSlider  : public Slider
+struct ColourComponentSlider final : public Slider
 {
     ColourComponentSlider (const String& name)  : Slider (name)
     {
@@ -45,7 +54,7 @@ struct ColourComponentSlider  : public Slider
 };
 
 //==============================================================================
-class ColourSelector::ColourSpaceView  : public Component
+class ColourSelector::ColourSpaceView final : public Component
 {
 public:
     ColourSpaceView (ColourSelector& cs, float& hue, float& sat, float& val, int edgeSize)
@@ -100,7 +109,7 @@ public:
 
     void updateIfNeeded()
     {
-        if (lastHue != h)
+        if (! approximatelyEqual (lastHue, h))
         {
             lastHue = h;
             colours = {};
@@ -125,7 +134,7 @@ private:
     const int edge;
     Image colours;
 
-    struct ColourSpaceMarker  : public Component
+    struct ColourSpaceMarker final : public Component
     {
         ColourSpaceMarker()
         {
@@ -156,7 +165,7 @@ private:
 };
 
 //==============================================================================
-class ColourSelector::HueSelectorComp  : public Component
+class ColourSelector::HueSelectorComp final : public Component
 {
 public:
     HueSelectorComp (ColourSelector& cs, float& hue, int edgeSize)
@@ -208,7 +217,7 @@ private:
     float& h;
     const int edge;
 
-    struct HueSelectorMarker  : public Component
+    struct HueSelectorMarker final : public Component
     {
         HueSelectorMarker()
         {
@@ -243,7 +252,7 @@ private:
 };
 
 //==============================================================================
-class ColourSelector::SwatchComponent   : public Component
+class ColourSelector::SwatchComponent final : public Component
 {
 public:
     SwatchComponent (ColourSelector& cs, int itemIndex)
@@ -263,9 +272,9 @@ public:
     void mouseDown (const MouseEvent&) override
     {
         PopupMenu m;
-        m.addItem (1, TRANS("Use this swatch as the current colour"));
+        m.addItem (1, TRANS ("Use this swatch as the current colour"));
         m.addSeparator();
-        m.addItem (2, TRANS("Set this swatch to the current colour"));
+        m.addItem (2, TRANS ("Set this swatch to the current colour"));
 
         m.showMenuAsync (PopupMenu::Options().withTargetComponent (this),
                          ModalCallbackFunction::forComponent (menuStaticCallback, this));
@@ -302,7 +311,7 @@ private:
 };
 
 //==============================================================================
-class ColourSelector::ColourPreviewComp  : public Component
+class ColourSelector::ColourPreviewComp final : public Component
 {
 public:
     ColourPreviewComp (ColourSelector& cs, bool isEditable)
@@ -373,7 +382,7 @@ private:
     ColourSelector& owner;
 
     Colour currentColour;
-    Font labelFont { 14.0f, Font::bold };
+    Font labelFont { withDefaultMetrics (FontOptions { 14.0f, Font::bold }) };
     int labelWidth = 0;
     Label colourLabel;
 
@@ -454,7 +463,7 @@ void ColourSelector::setHue (float newH)
 {
     newH = jlimit (0.0f, 1.0f, newH);
 
-    if (h != newH)
+    if (! approximatelyEqual (h, newH))
     {
         h = newH;
         colour = Colour (h, s, v, colour.getFloatAlpha());
@@ -467,7 +476,7 @@ void ColourSelector::setSV (float newS, float newV)
     newS = jlimit (0.0f, 1.0f, newS);
     newV = jlimit (0.0f, 1.0f, newV);
 
-    if (s != newS || v != newV)
+    if (! approximatelyEqual (s, newS) || ! approximatelyEqual (v, newV))
     {
         s = newS;
         v = newV;
@@ -598,7 +607,7 @@ void ColourSelector::resized()
 
         for (int i = 0; i < swatchComponents.size(); ++i)
         {
-            auto* sc = swatchComponents.getUnchecked(i);
+            auto* sc = swatchComponents.getUnchecked (i);
 
             sc->setBounds (x + xGap / 2,
                            y + yGap / 2,
